# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.formula_and_function_apm_resource_stats_data_source import (
        FormulaAndFunctionApmResourceStatsDataSource,
    )
    from datadog_api_client.v1.model.formula_and_function_apm_resource_stat_name import (
        FormulaAndFunctionApmResourceStatName,
    )


class FormulaAndFunctionApmResourceStatsQueryDefinition(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.formula_and_function_apm_resource_stats_data_source import (
            FormulaAndFunctionApmResourceStatsDataSource,
        )
        from datadog_api_client.v1.model.formula_and_function_apm_resource_stat_name import (
            FormulaAndFunctionApmResourceStatName,
        )

        return {
            "data_source": (FormulaAndFunctionApmResourceStatsDataSource,),
            "env": (str,),
            "group_by": ([str],),
            "name": (str,),
            "operation_name": (str,),
            "primary_tag_name": (str,),
            "primary_tag_value": (str,),
            "resource_name": (str,),
            "service": (str,),
            "stat": (FormulaAndFunctionApmResourceStatName,),
        }

    attribute_map = {
        "data_source": "data_source",
        "env": "env",
        "group_by": "group_by",
        "name": "name",
        "operation_name": "operation_name",
        "primary_tag_name": "primary_tag_name",
        "primary_tag_value": "primary_tag_value",
        "resource_name": "resource_name",
        "service": "service",
        "stat": "stat",
    }

    def __init__(
        self_,
        data_source: FormulaAndFunctionApmResourceStatsDataSource,
        env: str,
        name: str,
        service: str,
        stat: FormulaAndFunctionApmResourceStatName,
        group_by: Union[List[str], UnsetType] = unset,
        operation_name: Union[str, UnsetType] = unset,
        primary_tag_name: Union[str, UnsetType] = unset,
        primary_tag_value: Union[str, UnsetType] = unset,
        resource_name: Union[str, UnsetType] = unset,
        **kwargs,
    ):
        """
        APM resource stats query using formulas and functions.

        :param data_source: Data source for APM resource stats queries.
        :type data_source: FormulaAndFunctionApmResourceStatsDataSource

        :param env: APM environment.
        :type env: str

        :param group_by: Array of fields to group results by.
        :type group_by: [str], optional

        :param name: Name of this query to use in formulas.
        :type name: str

        :param operation_name: Name of operation on service.
        :type operation_name: str, optional

        :param primary_tag_name: Name of the second primary tag used within APM. Required when ``primary_tag_value`` is specified. See https://docs.datadoghq.com/tracing/guide/setting_primary_tags_to_scope/#add-a-second-primary-tag-in-datadog
        :type primary_tag_name: str, optional

        :param primary_tag_value: Value of the second primary tag by which to filter APM data. ``primary_tag_name`` must also be specified.
        :type primary_tag_value: str, optional

        :param resource_name: APM resource name.
        :type resource_name: str, optional

        :param service: APM service name.
        :type service: str

        :param stat: APM resource stat name.
        :type stat: FormulaAndFunctionApmResourceStatName
        """
        if group_by is not unset:
            kwargs["group_by"] = group_by
        if operation_name is not unset:
            kwargs["operation_name"] = operation_name
        if primary_tag_name is not unset:
            kwargs["primary_tag_name"] = primary_tag_name
        if primary_tag_value is not unset:
            kwargs["primary_tag_value"] = primary_tag_value
        if resource_name is not unset:
            kwargs["resource_name"] = resource_name
        super().__init__(kwargs)

        self_.data_source = data_source
        self_.env = env
        self_.name = name
        self_.service = service
        self_.stat = stat



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class LogsStorageTier(ModelSimple):
    """
    Specifies storage type as indexes or online-archives

    :param value: If omitted defaults to "indexes". Must be one of ["indexes", "online-archives"].
    :type value: str
    """

    allowed_values = {
        "indexes",
        "online-archives",
    }
    INDEXES: ClassVar["LogsStorageTier"]
    ONLINE_ARCHIVES: ClassVar["LogsStorageTier"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


LogsStorageTier.INDEXES = LogsStorageTier("indexes")
LogsStorageTier.ONLINE_ARCHIVES = LogsStorageTier("online-archives")



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    datetime,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.synthetics_ssl_certificate_issuer import SyntheticsSSLCertificateIssuer
    from datadog_api_client.v1.model.synthetics_ssl_certificate_subject import SyntheticsSSLCertificateSubject


class SyntheticsSSLCertificate(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.synthetics_ssl_certificate_issuer import SyntheticsSSLCertificateIssuer
        from datadog_api_client.v1.model.synthetics_ssl_certificate_subject import SyntheticsSSLCertificateSubject

        return {
            "cipher": (str,),
            "exponent": (float,),
            "ext_key_usage": ([str],),
            "fingerprint": (str,),
            "fingerprint256": (str,),
            "issuer": (SyntheticsSSLCertificateIssuer,),
            "modulus": (str,),
            "protocol": (str,),
            "serial_number": (str,),
            "subject": (SyntheticsSSLCertificateSubject,),
            "valid_from": (datetime,),
            "valid_to": (datetime,),
        }

    attribute_map = {
        "cipher": "cipher",
        "exponent": "exponent",
        "ext_key_usage": "extKeyUsage",
        "fingerprint": "fingerprint",
        "fingerprint256": "fingerprint256",
        "issuer": "issuer",
        "modulus": "modulus",
        "protocol": "protocol",
        "serial_number": "serialNumber",
        "subject": "subject",
        "valid_from": "validFrom",
        "valid_to": "validTo",
    }

    def __init__(
        self_,
        cipher: Union[str, UnsetType] = unset,
        exponent: Union[float, UnsetType] = unset,
        ext_key_usage: Union[List[str], UnsetType] = unset,
        fingerprint: Union[str, UnsetType] = unset,
        fingerprint256: Union[str, UnsetType] = unset,
        issuer: Union[SyntheticsSSLCertificateIssuer, UnsetType] = unset,
        modulus: Union[str, UnsetType] = unset,
        protocol: Union[str, UnsetType] = unset,
        serial_number: Union[str, UnsetType] = unset,
        subject: Union[SyntheticsSSLCertificateSubject, UnsetType] = unset,
        valid_from: Union[datetime, UnsetType] = unset,
        valid_to: Union[datetime, UnsetType] = unset,
        **kwargs,
    ):
        """
        Object describing the SSL certificate used for a Synthetic test.

        :param cipher: Cipher used for the connection.
        :type cipher: str, optional

        :param exponent: Exponent associated to the certificate.
        :type exponent: float, optional

        :param ext_key_usage: Array of extensions and details used for the certificate.
        :type ext_key_usage: [str], optional

        :param fingerprint: MD5 digest of the DER-encoded Certificate information.
        :type fingerprint: str, optional

        :param fingerprint256: SHA-1 digest of the DER-encoded Certificate information.
        :type fingerprint256: str, optional

        :param issuer: Object describing the issuer of a SSL certificate.
        :type issuer: SyntheticsSSLCertificateIssuer, optional

        :param modulus: Modulus associated to the SSL certificate private key.
        :type modulus: str, optional

        :param protocol: TLS protocol used for the test.
        :type protocol: str, optional

        :param serial_number: Serial Number assigned by Symantec to the SSL certificate.
        :type serial_number: str, optional

        :param subject: Object describing the SSL certificate used for the test.
        :type subject: SyntheticsSSLCertificateSubject, optional

        :param valid_from: Date from which the SSL certificate is valid.
        :type valid_from: datetime, optional

        :param valid_to: Date until which the SSL certificate is valid.
        :type valid_to: datetime, optional
        """
        if cipher is not unset:
            kwargs["cipher"] = cipher
        if exponent is not unset:
            kwargs["exponent"] = exponent
        if ext_key_usage is not unset:
            kwargs["ext_key_usage"] = ext_key_usage
        if fingerprint is not unset:
            kwargs["fingerprint"] = fingerprint
        if fingerprint256 is not unset:
            kwargs["fingerprint256"] = fingerprint256
        if issuer is not unset:
            kwargs["issuer"] = issuer
        if modulus is not unset:
            kwargs["modulus"] = modulus
        if protocol is not unset:
            kwargs["protocol"] = protocol
        if serial_number is not unset:
            kwargs["serial_number"] = serial_number
        if subject is not unset:
            kwargs["subject"] = subject
        if valid_from is not unset:
            kwargs["valid_from"] = valid_from
        if valid_to is not unset:
            kwargs["valid_to"] = valid_to
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class WidgetTimeWindows(ModelSimple):
    """
    Define a time window.

    :param value: Must be one of ["7d", "30d", "90d", "week_to_date", "previous_week", "month_to_date", "previous_month", "global_time"].
    :type value: str
    """

    allowed_values = {
        "7d",
        "30d",
        "90d",
        "week_to_date",
        "previous_week",
        "month_to_date",
        "previous_month",
        "global_time",
    }
    SEVEN_DAYS: ClassVar["WidgetTimeWindows"]
    THIRTY_DAYS: ClassVar["WidgetTimeWindows"]
    NINETY_DAYS: ClassVar["WidgetTimeWindows"]
    WEEK_TO_DATE: ClassVar["WidgetTimeWindows"]
    PREVIOUS_WEEK: ClassVar["WidgetTimeWindows"]
    MONTH_TO_DATE: ClassVar["WidgetTimeWindows"]
    PREVIOUS_MONTH: ClassVar["WidgetTimeWindows"]
    GLOBAL_TIME: ClassVar["WidgetTimeWindows"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


WidgetTimeWindows.SEVEN_DAYS = WidgetTimeWindows("7d")
WidgetTimeWindows.THIRTY_DAYS = WidgetTimeWindows("30d")
WidgetTimeWindows.NINETY_DAYS = WidgetTimeWindows("90d")
WidgetTimeWindows.WEEK_TO_DATE = WidgetTimeWindows("week_to_date")
WidgetTimeWindows.PREVIOUS_WEEK = WidgetTimeWindows("previous_week")
WidgetTimeWindows.MONTH_TO_DATE = WidgetTimeWindows("month_to_date")
WidgetTimeWindows.PREVIOUS_MONTH = WidgetTimeWindows("previous_month")
WidgetTimeWindows.GLOBAL_TIME = WidgetTimeWindows("global_time")



"""
Create a monitor returns "OK" response
"""

from os import environ
from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.monitors_api import MonitorsApi
from datadog_api_client.v1.model.monitor import Monitor
from datadog_api_client.v1.model.monitor_type import MonitorType

# there is a valid "role" in the system
ROLE_DATA_ID = environ["ROLE_DATA_ID"]

body = Monitor(
    name="Example-Create_a_monitor_returns_OK_response",
    type=MonitorType.LOG_ALERT,
    query='logs("service:foo AND type:error").index("main").rollup("count").by("source").last("5m") > 2',
    message="some message Notify: @hipchat-channel",
    tags=[
        "test:examplecreateamonitorreturnsokresponse",
        "env:ci",
    ],
    priority=3,
    restricted_roles=[
        ROLE_DATA_ID,
    ],
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = MonitorsApi(api_client)
    response = api_instance.create_monitor(body=body)

    print(response)



"""
Tag Configuration Cardinality Estimator returns "Success" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.metrics_api import MetricsApi

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = MetricsApi(api_client)
    response = api_instance.estimate_metrics_output_series(
        metric_name="system.cpu.idle",
        filter_groups="app,host",
        filter_num_aggregations=4,
    )

    print(response)



"""Data formatter."""
from collections import defaultdict
import json
from functools import singledispatch
import pathlib
import keyword
import warnings
import re

import dateutil.parser
import m2r2


MODEL_IMPORT_TPL = "datadog_api_client.{version}.model.{name}"
PRIMITIVE_TYPES = ["string", "number", "boolean", "integer"]

EDGE_CASES = {}
replacement_file = pathlib.Path(__file__).parent / "replacement.json"
if replacement_file.exists():
    with replacement_file.open() as f:
        EDGE_CASES.update(json.load(f))

KEYWORDS = set(keyword.kwlist)
KEYWORDS.add("property")
KEYWORDS.add("cls")

PATTERN_DOUBLE_UNDERSCORE = re.compile(r"__+")
PATTERN_LEADING_ALPHA = re.compile(r"(.)([A-Z][a-z]+)")
PATTERN_FOLLOWING_ALPHA = re.compile(r"([a-z0-9])([A-Z])")
PATTERN_WHITESPACE = re.compile(r"\W")


def snake_case(value):
    s1 = PATTERN_LEADING_ALPHA.sub(r"\1_\2", value)
    s1 = PATTERN_FOLLOWING_ALPHA.sub(r"\1_\2", s1).lower()
    s1 = PATTERN_WHITESPACE.sub("_", s1)
    s1 = s1.rstrip("_")
    return PATTERN_DOUBLE_UNDERSCORE.sub("_", s1)


def safe_snake_case(value):
    for token, replacement in EDGE_CASES.items():
        value = value.replace(token, replacement)
    return snake_case(value)


def camel_case(value):
    return "".join(x.title() for x in snake_case(value).split("_"))


def escape_reserved_keyword(word):
    """
    Escape reserved language keywords like openapi generator does it
    :param word: Word to escape
    :return: The escaped word if it was a reserved keyword, the word unchanged otherwise
    """
    if word in KEYWORDS:
        return f"_{word}"
    return word


def attribute_name(attribute):
    return escape_reserved_keyword(snake_case(attribute))


def format_value(value, quotes='"'):
    if isinstance(value, str):
        return f"{quotes}{value}{quotes}"
    elif isinstance(value, bool):
        return "true" if value else "false"
    return value


def get_name(schema):
    if hasattr(schema, "__reference__"):
        return schema.__reference__["$ref"].split("/")[-1]


def attribute_path(attribute):
    return ".".join(attribute_name(a) for a in attribute.split("."))


class CustomRenderer(m2r2.RestRenderer):
    def double_emphasis(self, text):
        if "``" in text:
            text = text.replace("\\ ``", "").replace("``\\ ", "")
        if "`_" in text:
            return text
        return "\\ **{}**\\ ".format(text)

    def header(self, text, level, raw=None):
        return "\n{}\n".format(self.double_emphasis(text))


def docstring(text):
    return (
        m2r2.convert(text.replace("\\n", "\\\\n"), renderer=CustomRenderer())[1:-1]
        .replace("\\ ", " ")
        .replace("\\`", "\\\\`")
        .replace("\n\n\n", "\n\n")
    )


def _merge_imports(a, b):
    """Merge second set of imports into first one."""
    for k, v in b.items():
        a[k] |= v
    return a


def format_parameters(kwargs, spec, version, replace_values=None):
    parameters = ""
    imports = defaultdict(set)

    parameters_spec = {p["name"]: p for p in spec.get("parameters", [])}
    if "requestBody" in spec and "multipart/form-data" in spec["requestBody"]["content"]:
        parent = spec["requestBody"]["content"]["multipart/form-data"]["schema"]
        for name, schema in parent["properties"].items():
            parameters_spec[name] = {
                "in": "form",
                "schema": schema,
                "name": name,
                "description": schema.get("description"),
                "required": name in parent.get("required", []),
            }

    parameters = ""
    for p in parameters_spec.values():
        k = p["name"]
        if k not in kwargs:
            continue

        v = kwargs[k]
        value, extra_imports = format_data_with_schema(
            v["value"],
            p["schema"],
            replace_values=replace_values,
            version=version,
        )
        imports = _merge_imports(imports, extra_imports)
        parameters += f"{escape_reserved_keyword(safe_snake_case(k))}={value}, "

    return parameters, imports


def get_name_and_imports(schema, version=None, imports=None):
    assert version is not None
    imports = imports or defaultdict(set)

    name = None
    if hasattr(schema, "__reference__"):
        name = schema.__reference__["$ref"].split("/")[-1]
        if "oneOf" not in schema:
            # do not include parent of oneOf schema
            imports[MODEL_IMPORT_TPL.format(version=version, name=safe_snake_case(name))].add(name)

    return name, imports


@singledispatch
def format_data_with_schema(
    data,
    schema,
    replace_values=None,
    default_name=None,
    version=None,
    imports=None,
):
    """Format data with schema."""
    assert version is not None

    name = None
    imports = imports or defaultdict(set)
    if schema.get("type") not in {"string", "integer", "boolean"} or schema.get("enum"):
        name, imports = get_name_and_imports(schema, version, imports)
    if schema.get("oneOf"):
        name = None
    if name:
        imports[MODEL_IMPORT_TPL.format(version=version, name=safe_snake_case(name))].add(name)

    if "enum" in schema and data not in schema["enum"]:
        raise ValueError(f"{data} is not valid enum value {schema['enum']}")

    if replace_values and data in replace_values:
        parameters = replace_values[data]
        if schema.get("format") in ("int32", "int64"):
            parameters = f"int({parameters})"
    elif "enum" in schema:
        parameters = schema["x-enum-varnames"][schema["enum"].index(data)]
        return f"{name}.{parameters}", imports
    else:
        if schema.get("nullable") and data is None:
            parameters = repr(data)
            return parameters, imports
        else:

            def format_datetime(x):
                imports["datetime"].add("datetime")
                d = dateutil.parser.isoparse(x)
                result = repr(d)
                if result.startswith("datetime."):
                    result = result[len("datetime.") :]
                if "tzutc" in result:
                    imports["dateutil.tz"].add("tzutc")
                if "tzoffset" in result:
                    imports["dateutil.tz"].add("tzoffset")
                return result

            formatter = {
                "double": lambda s: repr(float(s)),
                "int32": lambda s: repr(int(s)),
                "int64": lambda s: repr(int(s)),
                "date": format_datetime,
                "date-time": format_datetime,
                "binary": lambda s: f'open("{s}", "rb")',
                "email": repr,
                None: repr,
            }[schema.get("format")]

            # TODO format date and datetime
            parameters = formatter(data)

    if name:
        return f"{name}({parameters})", imports

    return parameters, imports


@format_data_with_schema.register(list)
def format_data_with_schema_list(
    data,
    schema,
    replace_values=None,
    default_name=None,
    version=None,
    imports=None,
):
    """Format data with schema."""
    assert version is not None
    name, imports = get_name_and_imports(schema, version, imports)

    if "oneOf" in schema:
        for sub_schema in schema["oneOf"]:
            try:
                value, one_of_imports = format_data_with_schema(
                    data,
                    sub_schema,
                    replace_values=replace_values,
                    version=version,
                )
            except (KeyError, ValueError):
                continue
            # Workaround to not generate schema for primitive nested oneOfs
            if sub_schema.get("items", {}).get("type") in PRIMITIVE_TYPES:
                return data, imports

            return value, one_of_imports
        raise ValueError(f"{data} is not valid oneOf {schema}")

    parameters = ""
    for d in data:
        value, extra_imports = format_data_with_schema(
            d,
            schema["items"],
            replace_values=replace_values,
            default_name=name,
            version=version,
        )
        parameters += f"{value}, "
        imports = _merge_imports(imports, extra_imports)
    parameters = f"[{parameters}]"

    if name:
        return f"{name}({parameters})", imports

    return parameters, imports


@format_data_with_schema.register(dict)
def format_data_with_schema_dict(
    data,
    schema,
    replace_values=None,
    default_name=None,
    version=None,
    imports=None,
):
    """Format data with schema."""
    assert version is not None
    name, imports = get_name_and_imports(schema, version, imports)

    parameters = ""
    if "properties" in schema:
        for k, v in data.items():
            if k in schema["properties"]:
                sub_schema = schema["properties"][k]
            else:
                sub_schema = schema["additionalProperties"]
            value, extra_imports = format_data_with_schema(
                v,
                sub_schema,
                replace_values=replace_values,
                default_name=name + camel_case(k) if name else None,
                version=version,
            )
            parameters += f"{escape_reserved_keyword(safe_snake_case(k))}={value}, "
            imports = _merge_imports(imports, extra_imports)

    if schema.get("additionalProperties") and not schema.get("properties"):
        for k, v in data.items():
            value, extra_imports = format_data_with_schema(
                v,
                schema["additionalProperties"],
                replace_values=replace_values,
                version=version,
            )
            parameters += f"{escape_reserved_keyword(k)}={value}, "
            imports = _merge_imports(imports, extra_imports)

    if not name and "oneOf" not in schema:
        if default_name and not schema.get("additionalProperties") and schema.get("properties"):
            name = default_name
            imports[MODEL_IMPORT_TPL.format(version=version, name=safe_snake_case(name))].add(name)
        else:
            name = "dict"
            warnings.warn(f"Unnamed schema {schema} for {data}")

    if "oneOf" in schema:
        matched = 0
        for sub_schema in schema["oneOf"]:
            try:
                formatted, extra_imports = format_data_with_schema(
                    data,
                    sub_schema,
                    replace_values=replace_values,
                    version=version,
                )
                if matched == 0:
                    imports = _merge_imports(imports, extra_imports)
                    # NOTE we do not support mixed schemas with oneOf
                    # parameters += formatted
                    parameters = formatted
                    name = None
                matched += 1
            except (KeyError, ValueError) as e:
                print(f"{e}")

        if matched == 0:
            raise ValueError(f"[{matched}] {data} is not valid for schema {name}")
        elif matched > 1:
            warnings.warn(f"[{matched}] {data} is not valid for schema {name}")

    if not parameters and data:
        parameters = ", ".join(f'{k}="{v}"' for k, v in data.items())

    if name:
        return f"{name}({parameters})", imports

    return parameters, imports



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.service_definition_v2_opsgenie_region import ServiceDefinitionV2OpsgenieRegion


class ServiceDefinitionV2Opsgenie(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.service_definition_v2_opsgenie_region import ServiceDefinitionV2OpsgenieRegion

        return {
            "region": (ServiceDefinitionV2OpsgenieRegion,),
            "service_url": (str,),
        }

    attribute_map = {
        "region": "region",
        "service_url": "service-url",
    }

    def __init__(
        self_, service_url: str, region: Union[ServiceDefinitionV2OpsgenieRegion, UnsetType] = unset, **kwargs
    ):
        """
        Opsgenie integration for the service.

        :param region: Opsgenie instance region.
        :type region: ServiceDefinitionV2OpsgenieRegion, optional

        :param service_url: Opsgenie service url.
        :type service_url: str
        """
        if region is not unset:
            kwargs["region"] = region
        super().__init__(kwargs)

        self_.service_url = service_url



"""
Update a security filter returns "OK" response
"""

from os import environ
from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.security_monitoring_api import SecurityMonitoringApi
from datadog_api_client.v2.model.security_filter_filtered_data_type import SecurityFilterFilteredDataType
from datadog_api_client.v2.model.security_filter_type import SecurityFilterType
from datadog_api_client.v2.model.security_filter_update_attributes import SecurityFilterUpdateAttributes
from datadog_api_client.v2.model.security_filter_update_data import SecurityFilterUpdateData
from datadog_api_client.v2.model.security_filter_update_request import SecurityFilterUpdateRequest

# there is a valid "security_filter" in the system
SECURITY_FILTER_DATA_ID = environ["SECURITY_FILTER_DATA_ID"]

body = SecurityFilterUpdateRequest(
    data=SecurityFilterUpdateData(
        attributes=SecurityFilterUpdateAttributes(
            exclusion_filters=[],
            filtered_data_type=SecurityFilterFilteredDataType.LOGS,
            is_enabled=True,
            name="Example-Update_a_security_filter_returns_OK_response",
            query="service:ExampleUpdateasecurityfilterreturnsOKresponse",
            version=1,
        ),
        type=SecurityFilterType.SECURITY_FILTERS,
    ),
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = SecurityMonitoringApi(api_client)
    response = api_instance.update_security_filter(security_filter_id=SECURITY_FILTER_DATA_ID, body=body)

    print(response)



"""
Delete an AuthN Mapping returns "OK" response
"""

from os import environ
from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.authn_mappings_api import AuthNMappingsApi

# there is a valid "authn_mapping" in the system
AUTHN_MAPPING_DATA_ID = environ["AUTHN_MAPPING_DATA_ID"]

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = AuthNMappingsApi(api_client)
    api_instance.delete_authn_mapping(
        authn_mapping_id=AUTHN_MAPPING_DATA_ID,
    )



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class SyntheticsBrowserTestRumSettings(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "application_id": (str,),
            "client_token_id": (int,),
            "is_enabled": (bool,),
        }

    attribute_map = {
        "application_id": "applicationId",
        "client_token_id": "clientTokenId",
        "is_enabled": "isEnabled",
    }

    def __init__(
        self_,
        is_enabled: bool,
        application_id: Union[str, UnsetType] = unset,
        client_token_id: Union[int, UnsetType] = unset,
        **kwargs,
    ):
        """
        The RUM data collection settings for the Synthetic browser test.
        **Note:** There are 3 ways to format RUM settings:

        ``{ isEnabled: false }``
        RUM data is not collected.

        ``{ isEnabled: true }``
        RUM data is collected from the Synthetic test's default application.

        ``{ isEnabled: true, applicationId: "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx", clientTokenId: 12345 }``
        RUM data is collected using the specified application.

        :param application_id: RUM application ID used to collect RUM data for the browser test.
        :type application_id: str, optional

        :param client_token_id: RUM application API key ID used to collect RUM data for the browser test.
        :type client_token_id: int, optional

        :param is_enabled: Determines whether RUM data is collected during test runs.
        :type is_enabled: bool
        """
        if application_id is not unset:
            kwargs["application_id"] = application_id
        if client_token_id is not unset:
            kwargs["client_token_id"] = client_token_id
        super().__init__(kwargs)

        self_.is_enabled = is_enabled



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelComposed,
    cached_property,
)


class RUMGroupByTotal(ModelComposed):
    def __init__(self, **kwargs):
        """
        A resulting object to put the given computes in over all the matching records.
        """
        super().__init__(kwargs)

    @cached_property
    def _composed_schemas(_):
        # we need this here to make our import statements work
        # we must store _composed_schemas in here so the code is only run
        # when we invoke this method. If we kept this at the class
        # level we would get an error because the class level
        # code would be run when this module is imported, and these composed
        # classes don't exist yet because their module has not finished
        # loading
        return {
            "oneOf": [
                bool,
                str,
                float,
            ],
        }



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    datetime,
    unset,
    UnsetType,
)


class LogsListRequestTime(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "_from": (datetime,),
            "timezone": (str,),
            "to": (datetime,),
        }

    attribute_map = {
        "_from": "from",
        "timezone": "timezone",
        "to": "to",
    }

    def __init__(self_, _from: datetime, to: datetime, timezone: Union[str, UnsetType] = unset, **kwargs):
        """
        Timeframe to retrieve the log from.

        :param _from: Minimum timestamp for requested logs.
        :type _from: datetime

        :param timezone: Timezone can be specified both as an offset (for example "UTC+03:00")
            or a regional zone (for example "Europe/Paris").
        :type timezone: str, optional

        :param to: Maximum timestamp for requested logs.
        :type to: datetime
        """
        if timezone is not unset:
            kwargs["timezone"] = timezone
        super().__init__(kwargs)

        self_._from = _from
        self_.to = to



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.metric_custom_space_aggregation import MetricCustomSpaceAggregation
    from datadog_api_client.v2.model.metric_custom_time_aggregation import MetricCustomTimeAggregation


class MetricCustomAggregation(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.metric_custom_space_aggregation import MetricCustomSpaceAggregation
        from datadog_api_client.v2.model.metric_custom_time_aggregation import MetricCustomTimeAggregation

        return {
            "space": (MetricCustomSpaceAggregation,),
            "time": (MetricCustomTimeAggregation,),
        }

    attribute_map = {
        "space": "space",
        "time": "time",
    }

    def __init__(self_, space: MetricCustomSpaceAggregation, time: MetricCustomTimeAggregation, **kwargs):
        """
        A time and space aggregation combination for use in query.

        :param space: A space aggregation for use in query.
        :type space: MetricCustomSpaceAggregation

        :param time: A time aggregation for use in query.
        :type time: MetricCustomTimeAggregation
        """
        super().__init__(kwargs)

        self_.space = space
        self_.time = time



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.security_monitoring_rule_severity import SecurityMonitoringRuleSeverity


class SecurityMonitoringRuleCase(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.security_monitoring_rule_severity import SecurityMonitoringRuleSeverity

        return {
            "condition": (str,),
            "name": (str,),
            "notifications": ([str],),
            "status": (SecurityMonitoringRuleSeverity,),
        }

    attribute_map = {
        "condition": "condition",
        "name": "name",
        "notifications": "notifications",
        "status": "status",
    }

    def __init__(
        self_,
        condition: Union[str, UnsetType] = unset,
        name: Union[str, UnsetType] = unset,
        notifications: Union[List[str], UnsetType] = unset,
        status: Union[SecurityMonitoringRuleSeverity, UnsetType] = unset,
        **kwargs,
    ):
        """
        Case when signal is generated.

        :param condition: A rule case contains logical operations ( ``>`` , ``>=`` , ``&&`` , ``||`` ) to determine if a signal should be generated
            based on the event counts in the previously defined queries.
        :type condition: str, optional

        :param name: Name of the case.
        :type name: str, optional

        :param notifications: Notification targets for each rule case.
        :type notifications: [str], optional

        :param status: Severity of the Security Signal.
        :type status: SecurityMonitoringRuleSeverity, optional
        """
        if condition is not unset:
            kwargs["condition"] = condition
        if name is not unset:
            kwargs["name"] = name
        if notifications is not unset:
            kwargs["notifications"] = notifications
        if status is not unset:
            kwargs["status"] = status
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed
# under the 3-clause BSD style license (see LICENSE).
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019 Datadog, Inc.

import os

from setuptools import setup

version_template = """\
# Unless explicitly stated otherwise all files in this repository are licensed
# under the 3-clause BSD style license (see LICENSE).
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2020-Present Datadog, Inc.

__version__ = "{version}"
"""

# Allows the fetching of tags even from shallow clones
# https://github.com/pypa/setuptools_scm/pull/118#issuecomment-255381535
def parse_fetch_on_shallow(root):
    from setuptools_scm.git import parse, fetch_on_shallow

    return parse(root, pre_parse=fetch_on_shallow)


setup(
    use_scm_version={
        "local_scheme": "dirty-tag",
        "write_to": os.path.join("src", "datadog_api_client", "version.py"),
        "write_to_template": version_template,
        "parse": parse_fetch_on_shallow,
    }
)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class LogsGrokParserRules(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "match_rules": (str,),
            "support_rules": (str,),
        }

    attribute_map = {
        "match_rules": "match_rules",
        "support_rules": "support_rules",
    }

    def __init__(self_, match_rules: str, support_rules: Union[str, UnsetType] = unset, **kwargs):
        """
        Set of rules for the grok parser.

        :param match_rules: List of match rules for the grok parser, separated by a new line.
        :type match_rules: str

        :param support_rules: List of support rules for the grok parser, separated by a new line.
        :type support_rules: str, optional
        """
        if support_rules is not unset:
            kwargs["support_rules"] = support_rules
        super().__init__(kwargs)

        self_.match_rules = match_rules



"""
Pause or start a test returns "OK - Returns a boolean indicating if the update was successful." response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.synthetics_api import SyntheticsApi
from datadog_api_client.v1.model.synthetics_test_pause_status import SyntheticsTestPauseStatus
from datadog_api_client.v1.model.synthetics_update_test_pause_status_payload import (
    SyntheticsUpdateTestPauseStatusPayload,
)

body = SyntheticsUpdateTestPauseStatusPayload(
    new_status=SyntheticsTestPauseStatus.LIVE,
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = SyntheticsApi(api_client)
    response = api_instance.update_test_pause_status(public_id="public_id", body=body)

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class CloudWorkloadSecurityAgentRuleCreateAttributes(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "description": (str,),
            "enabled": (bool,),
            "expression": (str,),
            "name": (str,),
        }

    attribute_map = {
        "description": "description",
        "enabled": "enabled",
        "expression": "expression",
        "name": "name",
    }

    def __init__(
        self_,
        expression: str,
        name: str,
        description: Union[str, UnsetType] = unset,
        enabled: Union[bool, UnsetType] = unset,
        **kwargs,
    ):
        """
        Create a new Cloud Workload Security Agent rule.

        :param description: The description of the Agent rule.
        :type description: str, optional

        :param enabled: Whether the Agent rule is enabled.
        :type enabled: bool, optional

        :param expression: The SECL expression of the Agent rule.
        :type expression: str

        :param name: The name of the Agent rule.
        :type name: str
        """
        if description is not unset:
            kwargs["description"] = description
        if enabled is not unset:
            kwargs["enabled"] = enabled
        super().__init__(kwargs)

        self_.expression = expression
        self_.name = name



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelComposed,
    cached_property,
)


class NotebookCellResponseAttributes(ModelComposed):
    def __init__(self, **kwargs):
        """
        The attributes of a notebook cell response. Valid cell types are ``markdown`` , ``timeseries`` , ``toplist`` , ``heatmap`` , ``distribution`` ,
        ``log_stream``. `More information on each graph visualization type. <https://docs.datadoghq.com/dashboards/widgets/>`_

        :param definition: Text in a notebook is formatted with [Markdown](https://daringfireball.net/projects/markdown/), which enables the use of headings, subheadings, links, images, lists, and code blocks.
        :type definition: NotebookMarkdownCellDefinition

        :param graph_size: The size of the graph.
        :type graph_size: NotebookGraphSize, optional

        :param split_by: Object describing how to split the graph to display multiple visualizations per request.
        :type split_by: NotebookSplitBy, optional

        :param time: Timeframe for the notebook cell. When 'null', the notebook global time is used.
        :type time: NotebookCellTime, none_type, optional
        """
        super().__init__(kwargs)

    @cached_property
    def _composed_schemas(_):
        # we need this here to make our import statements work
        # we must store _composed_schemas in here so the code is only run
        # when we invoke this method. If we kept this at the class
        # level we would get an error because the class level
        # code would be run when this module is imported, and these composed
        # classes don't exist yet because their module has not finished
        # loading
        from datadog_api_client.v1.model.notebook_markdown_cell_attributes import NotebookMarkdownCellAttributes
        from datadog_api_client.v1.model.notebook_timeseries_cell_attributes import NotebookTimeseriesCellAttributes
        from datadog_api_client.v1.model.notebook_toplist_cell_attributes import NotebookToplistCellAttributes
        from datadog_api_client.v1.model.notebook_heat_map_cell_attributes import NotebookHeatMapCellAttributes
        from datadog_api_client.v1.model.notebook_distribution_cell_attributes import NotebookDistributionCellAttributes
        from datadog_api_client.v1.model.notebook_log_stream_cell_attributes import NotebookLogStreamCellAttributes

        return {
            "oneOf": [
                NotebookMarkdownCellAttributes,
                NotebookTimeseriesCellAttributes,
                NotebookToplistCellAttributes,
                NotebookHeatMapCellAttributes,
                NotebookDistributionCellAttributes,
                NotebookLogStreamCellAttributes,
            ],
        }



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
)


class UsageAttributionTagNames(ModelNormal):
    @cached_property
    def additional_properties_type(_):
        return ([str],)

    def __init__(self_, **kwargs):
        """
        Tag keys and values.

        A ``null`` value here means that the requested tag breakdown cannot be applied because it does not match the `tags
        configured for usage attribution <https://docs.datadoghq.com/account_management/billing/usage_attribution/#getting-started>`_.
        In this scenario the API returns the total usage, not broken down by tags.
        """
        super().__init__(kwargs)



"""
List IP Ranges returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.ip_ranges_api import IPRangesApi

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = IPRangesApi(api_client)
    response = api_instance.get_ip_ranges()

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Any, Dict

from datadog_api_client.api_client import ApiClient, Endpoint as _Endpoint
from datadog_api_client.v1.model.azure_account import AzureAccount
from datadog_api_client.v1.model.azure_account_list_response import AzureAccountListResponse


class AzureIntegrationApi:
    """
    Configure your Datadog-Azure integration directly through the Datadog API.
    For more information, see the `Datadog-Azure integration page <https://docs.datadoghq.com/integrations/azure>`_.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        self._create_azure_integration_endpoint = _Endpoint(
            settings={
                "response_type": (dict,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v1/integration/azure",
                "operation_id": "create_azure_integration",
                "http_method": "POST",
                "version": "v1",
                "servers": None,
            },
            params_map={
                "body": {
                    "required": True,
                    "openapi_types": (AzureAccount,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

        self._delete_azure_integration_endpoint = _Endpoint(
            settings={
                "response_type": (dict,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v1/integration/azure",
                "operation_id": "delete_azure_integration",
                "http_method": "DELETE",
                "version": "v1",
                "servers": None,
            },
            params_map={
                "body": {
                    "required": True,
                    "openapi_types": (AzureAccount,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

        self._list_azure_integration_endpoint = _Endpoint(
            settings={
                "response_type": (AzureAccountListResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v1/integration/azure",
                "operation_id": "list_azure_integration",
                "http_method": "GET",
                "version": "v1",
                "servers": None,
            },
            params_map={},
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._update_azure_host_filters_endpoint = _Endpoint(
            settings={
                "response_type": (dict,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v1/integration/azure/host_filters",
                "operation_id": "update_azure_host_filters",
                "http_method": "POST",
                "version": "v1",
                "servers": None,
            },
            params_map={
                "body": {
                    "required": True,
                    "openapi_types": (AzureAccount,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

        self._update_azure_integration_endpoint = _Endpoint(
            settings={
                "response_type": (dict,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v1/integration/azure",
                "operation_id": "update_azure_integration",
                "http_method": "PUT",
                "version": "v1",
                "servers": None,
            },
            params_map={
                "body": {
                    "required": True,
                    "openapi_types": (AzureAccount,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

    def create_azure_integration(
        self,
        body: AzureAccount,
    ) -> dict:
        """Create an Azure integration.

        Create a Datadog-Azure integration.

        Using the ``POST`` method updates your integration configuration by adding your new
        configuration to the existing one in your Datadog organization.

        Using the ``PUT`` method updates your integration configuration by replacing your
        current configuration with the new one sent to your Datadog organization.

        :param body: Create a Datadog-Azure integration for your Datadog account request body.
        :type body: AzureAccount
        :rtype: dict
        """
        kwargs: Dict[str, Any] = {}
        kwargs["body"] = body

        return self._create_azure_integration_endpoint.call_with_http_info(**kwargs)

    def delete_azure_integration(
        self,
        body: AzureAccount,
    ) -> dict:
        """Delete an Azure integration.

        Delete a given Datadog-Azure integration from your Datadog account.

        :param body: Delete a given Datadog-Azure integration request body.
        :type body: AzureAccount
        :rtype: dict
        """
        kwargs: Dict[str, Any] = {}
        kwargs["body"] = body

        return self._delete_azure_integration_endpoint.call_with_http_info(**kwargs)

    def list_azure_integration(
        self,
    ) -> AzureAccountListResponse:
        """List all Azure integrations.

        List all Datadog-Azure integrations configured in your Datadog account.

        :rtype: AzureAccountListResponse
        """
        kwargs: Dict[str, Any] = {}
        return self._list_azure_integration_endpoint.call_with_http_info(**kwargs)

    def update_azure_host_filters(
        self,
        body: AzureAccount,
    ) -> dict:
        """Update Azure integration host filters.

        Update the defined list of host filters for a given Datadog-Azure integration.

        :param body: Update a Datadog-Azure integration's host filters request body.
        :type body: AzureAccount
        :rtype: dict
        """
        kwargs: Dict[str, Any] = {}
        kwargs["body"] = body

        return self._update_azure_host_filters_endpoint.call_with_http_info(**kwargs)

    def update_azure_integration(
        self,
        body: AzureAccount,
    ) -> dict:
        """Update an Azure integration.

        Update a Datadog-Azure integration. Requires an existing ``tenant_name`` and ``client_id``.
        Any other fields supplied will overwrite existing values. To overwrite ``tenant_name`` or ``client_id`` ,
        use ``new_tenant_name`` and ``new_client_id``. To leave a field unchanged, do not supply that field in the payload.

        :param body: Update a Datadog-Azure integration request body.
        :type body: AzureAccount
        :rtype: dict
        """
        kwargs: Dict[str, Any] = {}
        kwargs["body"] = body

        return self._update_azure_integration_endpoint.call_with_http_info(**kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.events_compute import EventsCompute
    from datadog_api_client.v2.model.events_data_source import EventsDataSource
    from datadog_api_client.v2.model.events_query_group_bys import EventsQueryGroupBys
    from datadog_api_client.v2.model.events_search import EventsSearch


class EventsScalarQuery(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.events_compute import EventsCompute
        from datadog_api_client.v2.model.events_data_source import EventsDataSource
        from datadog_api_client.v2.model.events_query_group_bys import EventsQueryGroupBys
        from datadog_api_client.v2.model.events_search import EventsSearch

        return {
            "compute": (EventsCompute,),
            "data_source": (EventsDataSource,),
            "group_by": (EventsQueryGroupBys,),
            "indexes": ([str],),
            "name": (str,),
            "search": (EventsSearch,),
        }

    attribute_map = {
        "compute": "compute",
        "data_source": "data_source",
        "group_by": "group_by",
        "indexes": "indexes",
        "name": "name",
        "search": "search",
    }

    def __init__(
        self_,
        compute: EventsCompute,
        data_source: EventsDataSource,
        group_by: Union[EventsQueryGroupBys, UnsetType] = unset,
        indexes: Union[List[str], UnsetType] = unset,
        name: Union[str, UnsetType] = unset,
        search: Union[EventsSearch, UnsetType] = unset,
        **kwargs,
    ):
        """
        An individual scalar events query.

        :param compute: The instructions for what to compute for this query.
        :type compute: EventsCompute

        :param data_source: A data source that is powered by the Events Platform.
        :type data_source: EventsDataSource

        :param group_by: The list of facets on which to split results.
        :type group_by: EventsQueryGroupBys, optional

        :param indexes: The indexes in which to search.
        :type indexes: [str], optional

        :param name: The variable name for use in formulas.
        :type name: str, optional

        :param search: Configuration of the search/filter for an events query.
        :type search: EventsSearch, optional
        """
        if group_by is not unset:
            kwargs["group_by"] = group_by
        if indexes is not unset:
            kwargs["indexes"] = indexes
        if name is not unset:
            kwargs["name"] = name
        if search is not unset:
            kwargs["search"] = search
        super().__init__(kwargs)

        self_.compute = compute
        self_.data_source = data_source



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class MonitorSearchResponseMetadata(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "page": (int,),
            "page_count": (int,),
            "per_page": (int,),
            "total_count": (int,),
        }

    attribute_map = {
        "page": "page",
        "page_count": "page_count",
        "per_page": "per_page",
        "total_count": "total_count",
    }
    read_only_vars = {
        "page",
        "page_count",
        "per_page",
        "total_count",
    }

    def __init__(
        self_,
        page: Union[int, UnsetType] = unset,
        page_count: Union[int, UnsetType] = unset,
        per_page: Union[int, UnsetType] = unset,
        total_count: Union[int, UnsetType] = unset,
        **kwargs,
    ):
        """
        Metadata about the response.

        :param page: The page to start paginating from.
        :type page: int, optional

        :param page_count: The number of pages.
        :type page_count: int, optional

        :param per_page: The number of monitors to return per page.
        :type per_page: int, optional

        :param total_count: The total number of monitors.
        :type total_count: int, optional
        """
        if page is not unset:
            kwargs["page"] = page
        if page_count is not unset:
            kwargs["page_count"] = page_count
        if per_page is not unset:
            kwargs["per_page"] = per_page
        if total_count is not unset:
            kwargs["total_count"] = total_count
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.signal_archive_reason import SignalArchiveReason
    from datadog_api_client.v1.model.signal_triage_state import SignalTriageState


class SignalStateUpdateRequest(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.signal_archive_reason import SignalArchiveReason
        from datadog_api_client.v1.model.signal_triage_state import SignalTriageState

        return {
            "archive_comment": (str,),
            "archive_reason": (SignalArchiveReason,),
            "state": (SignalTriageState,),
            "version": (int,),
        }

    attribute_map = {
        "archive_comment": "archiveComment",
        "archive_reason": "archiveReason",
        "state": "state",
        "version": "version",
    }

    def __init__(
        self_,
        state: SignalTriageState,
        archive_comment: Union[str, UnsetType] = unset,
        archive_reason: Union[SignalArchiveReason, UnsetType] = unset,
        version: Union[int, UnsetType] = unset,
        **kwargs,
    ):
        """
        Attributes describing the change of state for a given state.

        :param archive_comment: Optional comment to explain why a signal is being archived.
        :type archive_comment: str, optional

        :param archive_reason: Reason why a signal has been archived.
        :type archive_reason: SignalArchiveReason, optional

        :param state: The new triage state of the signal.
        :type state: SignalTriageState

        :param version: Version of the updated signal. If server side version is higher, update will be rejected.
        :type version: int, optional
        """
        if archive_comment is not unset:
            kwargs["archive_comment"] = archive_comment
        if archive_reason is not unset:
            kwargs["archive_reason"] = archive_reason
        if version is not unset:
            kwargs["version"] = version
        super().__init__(kwargs)

        self_.state = state



"""
Create a cloud_configuration rule returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.security_monitoring_api import SecurityMonitoringApi
from datadog_api_client.v2.model.cloud_configuration_compliance_rule_options import (
    CloudConfigurationComplianceRuleOptions,
)
from datadog_api_client.v2.model.cloud_configuration_rego_rule import CloudConfigurationRegoRule
from datadog_api_client.v2.model.cloud_configuration_rule_case_create import CloudConfigurationRuleCaseCreate
from datadog_api_client.v2.model.cloud_configuration_rule_compliance_signal_options import (
    CloudConfigurationRuleComplianceSignalOptions,
)
from datadog_api_client.v2.model.cloud_configuration_rule_create_payload import CloudConfigurationRuleCreatePayload
from datadog_api_client.v2.model.cloud_configuration_rule_options import CloudConfigurationRuleOptions
from datadog_api_client.v2.model.cloud_configuration_rule_type import CloudConfigurationRuleType
from datadog_api_client.v2.model.security_monitoring_rule_severity import SecurityMonitoringRuleSeverity

body = CloudConfigurationRuleCreatePayload(
    type=CloudConfigurationRuleType.CLOUD_CONFIGURATION,
    name="Example-Create_a_cloud_configuration_rule_returns_OK_response_cloud",
    is_enabled=False,
    cases=[
        CloudConfigurationRuleCaseCreate(
            status=SecurityMonitoringRuleSeverity.INFO,
            notifications=[
                "channel",
            ],
        ),
    ],
    options=CloudConfigurationRuleOptions(
        compliance_rule_options=CloudConfigurationComplianceRuleOptions(
            resource_type="gcp_compute_disk",
            complex_rule=False,
            rego_rule=CloudConfigurationRegoRule(
                policy="package datadog\n",
                resource_types=[
                    "gcp_compute_disk",
                ],
            ),
        ),
    ),
    message="ddd",
    tags=[
        "my:tag",
    ],
    compliance_signal_options=CloudConfigurationRuleComplianceSignalOptions(
        user_activation_status=True,
        user_group_by_fields=[
            "@account_id",
        ],
    ),
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = SecurityMonitoringApi(api_client)
    response = api_instance.create_security_monitoring_rule(body=body)

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class SignalArchiveReason(ModelSimple):
    """
    Reason why a signal has been archived.

    :param value: Must be one of ["none", "false_positive", "testing_or_maintenance", "other"].
    :type value: str
    """

    allowed_values = {
        "none",
        "false_positive",
        "testing_or_maintenance",
        "other",
    }
    NONE: ClassVar["SignalArchiveReason"]
    FALSE_POSITIVE: ClassVar["SignalArchiveReason"]
    TESTING_OR_MAINTENANCE: ClassVar["SignalArchiveReason"]
    OTHER: ClassVar["SignalArchiveReason"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


SignalArchiveReason.NONE = SignalArchiveReason("none")
SignalArchiveReason.FALSE_POSITIVE = SignalArchiveReason("false_positive")
SignalArchiveReason.TESTING_OR_MAINTENANCE = SignalArchiveReason("testing_or_maintenance")
SignalArchiveReason.OTHER = SignalArchiveReason("other")



"""
Delete an application key owned by current user returns "No Content" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.key_management_api import KeyManagementApi

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = KeyManagementApi(api_client)
    api_instance.delete_current_user_application_key(
        app_key_id="app_key_id",
    )



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.monitor_search_count import MonitorSearchCount


class MonitorSearchResponseCounts(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.monitor_search_count import MonitorSearchCount

        return {
            "muted": (MonitorSearchCount,),
            "status": (MonitorSearchCount,),
            "tag": (MonitorSearchCount,),
            "type": (MonitorSearchCount,),
        }

    attribute_map = {
        "muted": "muted",
        "status": "status",
        "tag": "tag",
        "type": "type",
    }

    def __init__(
        self_,
        muted: Union[MonitorSearchCount, UnsetType] = unset,
        status: Union[MonitorSearchCount, UnsetType] = unset,
        tag: Union[MonitorSearchCount, UnsetType] = unset,
        type: Union[MonitorSearchCount, UnsetType] = unset,
        **kwargs,
    ):
        """
        The counts of monitors per different criteria.

        :param muted: Search facets.
        :type muted: MonitorSearchCount, optional

        :param status: Search facets.
        :type status: MonitorSearchCount, optional

        :param tag: Search facets.
        :type tag: MonitorSearchCount, optional

        :param type: Search facets.
        :type type: MonitorSearchCount, optional
        """
        if muted is not unset:
            kwargs["muted"] = muted
        if status is not unset:
            kwargs["status"] = status
        if tag is not unset:
            kwargs["tag"] = tag
        if type is not unset:
            kwargs["type"] = type
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class AuthNMappingUpdateAttributes(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "attribute_key": (str,),
            "attribute_value": (str,),
        }

    attribute_map = {
        "attribute_key": "attribute_key",
        "attribute_value": "attribute_value",
    }

    def __init__(
        self_, attribute_key: Union[str, UnsetType] = unset, attribute_value: Union[str, UnsetType] = unset, **kwargs
    ):
        """
        Key/Value pair of attributes used for update request.

        :param attribute_key: Key portion of a key/value pair of the attribute sent from the Identity Provider.
        :type attribute_key: str, optional

        :param attribute_value: Value portion of a key/value pair of the attribute sent from the Identity Provider.
        :type attribute_value: str, optional
        """
        if attribute_key is not unset:
            kwargs["attribute_key"] = attribute_key
        if attribute_value is not unset:
            kwargs["attribute_value"] = attribute_value
        super().__init__(kwargs)



"""
Create an AuthN Mapping returns "OK" response
"""

from os import environ
from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.authn_mappings_api import AuthNMappingsApi
from datadog_api_client.v2.model.authn_mapping_create_attributes import AuthNMappingCreateAttributes
from datadog_api_client.v2.model.authn_mapping_create_data import AuthNMappingCreateData
from datadog_api_client.v2.model.authn_mapping_create_relationships import AuthNMappingCreateRelationships
from datadog_api_client.v2.model.authn_mapping_create_request import AuthNMappingCreateRequest
from datadog_api_client.v2.model.authn_mappings_type import AuthNMappingsType
from datadog_api_client.v2.model.relationship_to_role import RelationshipToRole
from datadog_api_client.v2.model.relationship_to_role_data import RelationshipToRoleData
from datadog_api_client.v2.model.roles_type import RolesType

# there is a valid "role" in the system
ROLE_DATA_ID = environ["ROLE_DATA_ID"]

body = AuthNMappingCreateRequest(
    data=AuthNMappingCreateData(
        attributes=AuthNMappingCreateAttributes(
            attribute_key="examplecreateanauthnmappingreturnsokresponse",
            attribute_value="Example-Create_an_AuthN_Mapping_returns_OK_response",
        ),
        relationships=AuthNMappingCreateRelationships(
            role=RelationshipToRole(
                data=RelationshipToRoleData(
                    id=ROLE_DATA_ID,
                    type=RolesType.ROLES,
                ),
            ),
        ),
        type=AuthNMappingsType.AUTHN_MAPPINGS,
    ),
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = AuthNMappingsApi(api_client)
    response = api_instance.create_authn_mapping(body=body)

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.widget_layout_type import WidgetLayoutType
    from datadog_api_client.v1.model.widget_text_align import WidgetTextAlign
    from datadog_api_client.v1.model.group_widget_definition_type import GroupWidgetDefinitionType
    from datadog_api_client.v1.model.widget import Widget


class GroupWidgetDefinition(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.widget_layout_type import WidgetLayoutType
        from datadog_api_client.v1.model.widget_text_align import WidgetTextAlign
        from datadog_api_client.v1.model.group_widget_definition_type import GroupWidgetDefinitionType
        from datadog_api_client.v1.model.widget import Widget

        return {
            "background_color": (str,),
            "banner_img": (str,),
            "layout_type": (WidgetLayoutType,),
            "show_title": (bool,),
            "title": (str,),
            "title_align": (WidgetTextAlign,),
            "type": (GroupWidgetDefinitionType,),
            "widgets": ([Widget],),
        }

    attribute_map = {
        "background_color": "background_color",
        "banner_img": "banner_img",
        "layout_type": "layout_type",
        "show_title": "show_title",
        "title": "title",
        "title_align": "title_align",
        "type": "type",
        "widgets": "widgets",
    }

    def __init__(
        self_,
        layout_type: WidgetLayoutType,
        type: GroupWidgetDefinitionType,
        widgets: List[Widget],
        background_color: Union[str, UnsetType] = unset,
        banner_img: Union[str, UnsetType] = unset,
        show_title: Union[bool, UnsetType] = unset,
        title: Union[str, UnsetType] = unset,
        title_align: Union[WidgetTextAlign, UnsetType] = unset,
        **kwargs,
    ):
        """
        The groups widget allows you to keep similar graphs together on your timeboard. Each group has a custom header, can hold one to many graphs, and is collapsible.

        :param background_color: Background color of the group title.
        :type background_color: str, optional

        :param banner_img: URL of image to display as a banner for the group.
        :type banner_img: str, optional

        :param layout_type: Layout type of the group.
        :type layout_type: WidgetLayoutType

        :param show_title: Whether to show the title or not.
        :type show_title: bool, optional

        :param title: Title of the widget.
        :type title: str, optional

        :param title_align: How to align the text on the widget.
        :type title_align: WidgetTextAlign, optional

        :param type: Type of the group widget.
        :type type: GroupWidgetDefinitionType

        :param widgets: List of widget groups.
        :type widgets: [Widget]
        """
        if background_color is not unset:
            kwargs["background_color"] = background_color
        if banner_img is not unset:
            kwargs["banner_img"] = banner_img
        if show_title is not unset:
            kwargs["show_title"] = show_title
        if title is not unset:
            kwargs["title"] = title
        if title_align is not unset:
            kwargs["title_align"] = title_align
        super().__init__(kwargs)

        self_.layout_type = layout_type
        self_.type = type
        self_.widgets = widgets



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    none_type,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.logs_archive_destination import LogsArchiveDestination
    from datadog_api_client.v2.model.logs_archive_state import LogsArchiveState
    from datadog_api_client.v2.model.logs_archive_destination_azure import LogsArchiveDestinationAzure
    from datadog_api_client.v2.model.logs_archive_destination_gcs import LogsArchiveDestinationGCS
    from datadog_api_client.v2.model.logs_archive_destination_s3 import LogsArchiveDestinationS3


class LogsArchiveAttributes(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.logs_archive_destination import LogsArchiveDestination
        from datadog_api_client.v2.model.logs_archive_state import LogsArchiveState

        return {
            "destination": (LogsArchiveDestination,),
            "include_tags": (bool,),
            "name": (str,),
            "query": (str,),
            "rehydration_max_scan_size_in_gb": (int, none_type),
            "rehydration_tags": ([str],),
            "state": (LogsArchiveState,),
        }

    attribute_map = {
        "destination": "destination",
        "include_tags": "include_tags",
        "name": "name",
        "query": "query",
        "rehydration_max_scan_size_in_gb": "rehydration_max_scan_size_in_gb",
        "rehydration_tags": "rehydration_tags",
        "state": "state",
    }

    def __init__(
        self_,
        destination: Union[
            Union[
                LogsArchiveDestination, LogsArchiveDestinationAzure, LogsArchiveDestinationGCS, LogsArchiveDestinationS3
            ],
            none_type,
        ],
        name: str,
        query: str,
        include_tags: Union[bool, UnsetType] = unset,
        rehydration_max_scan_size_in_gb: Union[int, none_type, UnsetType] = unset,
        rehydration_tags: Union[List[str], UnsetType] = unset,
        state: Union[LogsArchiveState, UnsetType] = unset,
        **kwargs,
    ):
        """
        The attributes associated with the archive.

        :param destination: An archive's destination.
        :type destination: LogsArchiveDestination, none_type

        :param include_tags: To store the tags in the archive, set the value "true".
            If it is set to "false", the tags will be deleted when the logs are sent to the archive.
        :type include_tags: bool, optional

        :param name: The archive name.
        :type name: str

        :param query: The archive query/filter. Logs matching this query are included in the archive.
        :type query: str

        :param rehydration_max_scan_size_in_gb: Maximum scan size for rehydration from this archive.
        :type rehydration_max_scan_size_in_gb: int, none_type, optional

        :param rehydration_tags: An array of tags to add to rehydrated logs from an archive.
        :type rehydration_tags: [str], optional

        :param state: The state of the archive.
        :type state: LogsArchiveState, optional
        """
        if include_tags is not unset:
            kwargs["include_tags"] = include_tags
        if rehydration_max_scan_size_in_gb is not unset:
            kwargs["rehydration_max_scan_size_in_gb"] = rehydration_max_scan_size_in_gb
        if rehydration_tags is not unset:
            kwargs["rehydration_tags"] = rehydration_tags
        if state is not unset:
            kwargs["state"] = state
        super().__init__(kwargs)

        self_.destination = destination
        self_.name = name
        self_.query = query



import os
import time

import pytest

from datadog_api_client.api_client import AsyncApiClient
from datadog_api_client.configuration import Configuration
from datadog_api_client.exceptions import ForbiddenException
from datadog_api_client.v1.api import dashboards_api, metrics_api


@pytest.mark.asyncio
async def test_error():
    configuration = Configuration()
    configuration.api_key["apiKeyAuth"] = "invalid"

    async with AsyncApiClient(configuration) as api_client:
        api_instance = metrics_api.MetricsApi(api_client)
        with pytest.raises(ForbiddenException) as e:
            await api_instance.get_metric_metadata("some_metric")
        error = str(e.value)
        assert "(403)" in error
        assert "Reason: Forbidden" in error
        assert e.value.headers["Content-Type"] == "application/json"
        assert e.value.body["errors"] == ["Forbidden"]


@pytest.mark.asyncio
async def test_basic():
    if os.getenv("RECORD", "false").lower() != "none":
        pytest.skip("Integration test")
    configuration = Configuration(return_http_data_only=False)
    configuration.api_key["apiKeyAuth"] = os.getenv("DD_TEST_CLIENT_API_KEY", "fake")
    configuration.api_key["appKeyAuth"] = os.getenv("DD_TEST_CLIENT_APP_KEY", "fake")
    configuration.debug = os.getenv("DEBUG") in {"true", "1", "yes", "on"}
    if "DD_TEST_SITE" in os.environ:
        configuration.server_index = 2
        configuration.server_variables["site"] = os.environ["DD_TEST_SITE"]

    async with AsyncApiClient(configuration) as api_client:
        api_instance = dashboards_api.DashboardsApi(api_client)
        _, code, headers = await api_instance.list_dashboards()
        assert code == 200
        assert headers["Content-Type"] == "application/json"


@pytest.mark.asyncio
async def test_body():
    if os.getenv("RECORD", "false").lower() != "none":
        pytest.skip("Integration test")
    configuration = Configuration(return_http_data_only=False)
    configuration.api_key["apiKeyAuth"] = os.getenv("DD_TEST_CLIENT_API_KEY", "fake")
    configuration.api_key["appKeyAuth"] = os.getenv("DD_TEST_CLIENT_APP_KEY", "fake")
    configuration.debug = os.getenv("DEBUG") in {"true", "1", "yes", "on"}
    if "DD_TEST_SITE" in os.environ:
        configuration.server_index = 2
        configuration.server_variables["site"] = os.environ["DD_TEST_SITE"]

    body = {
        "series": [
            {
                "metric": "system.load.1",
                "points": [[time.time(), 0.7]],
                "tags": ["test:async_test"],
            },
        ]
    }

    async with AsyncApiClient(configuration) as api_client:
        api_instance = metrics_api.MetricsApi(api_client)
        _, code, headers = await api_instance.submit_metrics(body=body)
        assert code == 202



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.slo_error_timeframe import SLOErrorTimeframe


class SLOBulkDeleteError(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.slo_error_timeframe import SLOErrorTimeframe

        return {
            "id": (str,),
            "message": (str,),
            "timeframe": (SLOErrorTimeframe,),
        }

    attribute_map = {
        "id": "id",
        "message": "message",
        "timeframe": "timeframe",
    }

    def __init__(self_, id: str, message: str, timeframe: SLOErrorTimeframe, **kwargs):
        """
        Object describing the error.

        :param id: The ID of the service level objective object associated with
            this error.
        :type id: str

        :param message: The error message.
        :type message: str

        :param timeframe: The timeframe of the threshold associated with this error
            or "all" if all thresholds are affected.
        :type timeframe: SLOErrorTimeframe
        """
        super().__init__(kwargs)

        self_.id = id
        self_.message = message
        self_.timeframe = timeframe



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    datetime,
    unset,
    UnsetType,
)


class NotebookAbsoluteTime(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "end": (datetime,),
            "live": (bool,),
            "start": (datetime,),
        }

    attribute_map = {
        "end": "end",
        "live": "live",
        "start": "start",
    }

    def __init__(self_, end: datetime, start: datetime, live: Union[bool, UnsetType] = unset, **kwargs):
        """
        Absolute timeframe.

        :param end: The end time.
        :type end: datetime

        :param live: Indicates whether the timeframe should be shifted to end at the current time.
        :type live: bool, optional

        :param start: The start time.
        :type start: datetime
        """
        if live is not unset:
            kwargs["live"] = live
        super().__init__(kwargs)

        self_.end = end
        self_.start = start



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class LogsMetricResponseComputeAggregationType(ModelSimple):
    """
    The type of aggregation to use.

    :param value: Must be one of ["count", "distribution"].
    :type value: str
    """

    allowed_values = {
        "count",
        "distribution",
    }
    COUNT: ClassVar["LogsMetricResponseComputeAggregationType"]
    DISTRIBUTION: ClassVar["LogsMetricResponseComputeAggregationType"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


LogsMetricResponseComputeAggregationType.COUNT = LogsMetricResponseComputeAggregationType("count")
LogsMetricResponseComputeAggregationType.DISTRIBUTION = LogsMetricResponseComputeAggregationType("distribution")



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class MetricOrigin(ModelNormal):
    validations = {
        "metric_type": {
            "inclusive_maximum": 1000,
        },
        "product": {
            "inclusive_maximum": 1000,
        },
        "service": {
            "inclusive_maximum": 1000,
        },
    }

    @cached_property
    def openapi_types(_):
        return {
            "metric_type": (int,),
            "product": (int,),
            "service": (int,),
        }

    attribute_map = {
        "metric_type": "metric_type",
        "product": "product",
        "service": "service",
    }

    def __init__(
        self_,
        metric_type: Union[int, UnsetType] = unset,
        product: Union[int, UnsetType] = unset,
        service: Union[int, UnsetType] = unset,
        **kwargs,
    ):
        """
        Metric origin information.

        :param metric_type: The origin metric type code
        :type metric_type: int, optional

        :param product: The origin product code
        :type product: int, optional

        :param service: The origin service code
        :type service: int, optional
        """
        if metric_type is not unset:
            kwargs["metric_type"] = metric_type
        if product is not unset:
            kwargs["product"] = product
        if service is not unset:
            kwargs["service"] = service
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    none_type,
    unset,
    UnsetType,
)


class DashboardTemplateVariable(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "available_values": ([str], none_type),
            "default": (str, none_type),
            "defaults": ([str],),
            "name": (str,),
            "prefix": (str, none_type),
        }

    attribute_map = {
        "available_values": "available_values",
        "default": "default",
        "defaults": "defaults",
        "name": "name",
        "prefix": "prefix",
    }

    def __init__(
        self_,
        name: str,
        available_values: Union[List[str], none_type, UnsetType] = unset,
        default: Union[str, none_type, UnsetType] = unset,
        defaults: Union[List[str], UnsetType] = unset,
        prefix: Union[str, none_type, UnsetType] = unset,
        **kwargs,
    ):
        """
        Template variable.

        :param available_values: The list of values that the template variable drop-down is limited to.
        :type available_values: [str], none_type, optional

        :param default: (deprecated) The default value for the template variable on dashboard load. Cannot be used in conjunction with ``defaults``. **Deprecated**.
        :type default: str, none_type, optional

        :param defaults: One or many default values for template variables on load. If more than one default is specified, they will be unioned together with ``OR``. Cannot be used in conjunction with ``default``.
        :type defaults: [str], optional

        :param name: The name of the variable.
        :type name: str

        :param prefix: The tag prefix associated with the variable. Only tags with this prefix appear in the variable drop-down.
        :type prefix: str, none_type, optional
        """
        if available_values is not unset:
            kwargs["available_values"] = available_values
        if default is not unset:
            kwargs["default"] = default
        if defaults is not unset:
            kwargs["defaults"] = defaults
        if prefix is not unset:
            kwargs["prefix"] = prefix
        super().__init__(kwargs)

        self_.name = name



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.synthetics_browser_test_result_short_result import (
        SyntheticsBrowserTestResultShortResult,
    )
    from datadog_api_client.v1.model.synthetics_test_monitor_status import SyntheticsTestMonitorStatus


class SyntheticsBrowserTestResultShort(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.synthetics_browser_test_result_short_result import (
            SyntheticsBrowserTestResultShortResult,
        )
        from datadog_api_client.v1.model.synthetics_test_monitor_status import SyntheticsTestMonitorStatus

        return {
            "check_time": (float,),
            "probe_dc": (str,),
            "result": (SyntheticsBrowserTestResultShortResult,),
            "result_id": (str,),
            "status": (SyntheticsTestMonitorStatus,),
        }

    attribute_map = {
        "check_time": "check_time",
        "probe_dc": "probe_dc",
        "result": "result",
        "result_id": "result_id",
        "status": "status",
    }

    def __init__(
        self_,
        check_time: Union[float, UnsetType] = unset,
        probe_dc: Union[str, UnsetType] = unset,
        result: Union[SyntheticsBrowserTestResultShortResult, UnsetType] = unset,
        result_id: Union[str, UnsetType] = unset,
        status: Union[SyntheticsTestMonitorStatus, UnsetType] = unset,
        **kwargs,
    ):
        """
        Object with the results of a single Synthetic browser test.

        :param check_time: Last time the browser test was performed.
        :type check_time: float, optional

        :param probe_dc: Location from which the Browser test was performed.
        :type probe_dc: str, optional

        :param result: Object with the result of the last browser test run.
        :type result: SyntheticsBrowserTestResultShortResult, optional

        :param result_id: ID of the browser test result.
        :type result_id: str, optional

        :param status: The status of your Synthetic monitor.

            * ``O`` for not triggered
            * ``1`` for triggered
            * ``2`` for no data
        :type status: SyntheticsTestMonitorStatus, optional
        """
        if check_time is not unset:
            kwargs["check_time"] = check_time
        if probe_dc is not unset:
            kwargs["probe_dc"] = probe_dc
        if result is not unset:
            kwargs["result"] = result
        if result_id is not unset:
            kwargs["result_id"] = result_id
        if status is not unset:
            kwargs["status"] = status
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Dict, List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    datetime,
    none_type,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.incident_field_attributes import IncidentFieldAttributes
    from datadog_api_client.v2.model.incident_notification_handle import IncidentNotificationHandle
    from datadog_api_client.v2.model.incident_field_attributes_single_value import IncidentFieldAttributesSingleValue
    from datadog_api_client.v2.model.incident_field_attributes_multiple_value import (
        IncidentFieldAttributesMultipleValue,
    )


class IncidentUpdateAttributes(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.incident_field_attributes import IncidentFieldAttributes
        from datadog_api_client.v2.model.incident_notification_handle import IncidentNotificationHandle

        return {
            "customer_impact_end": (datetime, none_type),
            "customer_impact_scope": (str,),
            "customer_impact_start": (datetime, none_type),
            "customer_impacted": (bool,),
            "detected": (datetime, none_type),
            "fields": ({str: (IncidentFieldAttributes,)},),
            "notification_handles": ([IncidentNotificationHandle],),
            "title": (str,),
        }

    attribute_map = {
        "customer_impact_end": "customer_impact_end",
        "customer_impact_scope": "customer_impact_scope",
        "customer_impact_start": "customer_impact_start",
        "customer_impacted": "customer_impacted",
        "detected": "detected",
        "fields": "fields",
        "notification_handles": "notification_handles",
        "title": "title",
    }

    def __init__(
        self_,
        customer_impact_end: Union[datetime, none_type, UnsetType] = unset,
        customer_impact_scope: Union[str, UnsetType] = unset,
        customer_impact_start: Union[datetime, none_type, UnsetType] = unset,
        customer_impacted: Union[bool, UnsetType] = unset,
        detected: Union[datetime, none_type, UnsetType] = unset,
        fields: Union[
            Dict[
                str,
                Union[
                    IncidentFieldAttributes, IncidentFieldAttributesSingleValue, IncidentFieldAttributesMultipleValue
                ],
            ],
            UnsetType,
        ] = unset,
        notification_handles: Union[List[IncidentNotificationHandle], UnsetType] = unset,
        title: Union[str, UnsetType] = unset,
        **kwargs,
    ):
        """
        The incident's attributes for an update request.

        :param customer_impact_end: Timestamp when customers were no longer impacted by the incident.
        :type customer_impact_end: datetime, none_type, optional

        :param customer_impact_scope: A summary of the impact customers experienced during the incident.
        :type customer_impact_scope: str, optional

        :param customer_impact_start: Timestamp when customers began being impacted by the incident.
        :type customer_impact_start: datetime, none_type, optional

        :param customer_impacted: A flag indicating whether the incident caused customer impact.
        :type customer_impacted: bool, optional

        :param detected: Timestamp when the incident was detected.
        :type detected: datetime, none_type, optional

        :param fields: A condensed view of the user-defined fields for which to update selections.
        :type fields: {str: (IncidentFieldAttributes,)}, optional

        :param notification_handles: Notification handles that will be notified of the incident during update.
        :type notification_handles: [IncidentNotificationHandle], optional

        :param title: The title of the incident, which summarizes what happened.
        :type title: str, optional
        """
        if customer_impact_end is not unset:
            kwargs["customer_impact_end"] = customer_impact_end
        if customer_impact_scope is not unset:
            kwargs["customer_impact_scope"] = customer_impact_scope
        if customer_impact_start is not unset:
            kwargs["customer_impact_start"] = customer_impact_start
        if customer_impacted is not unset:
            kwargs["customer_impacted"] = customer_impacted
        if detected is not unset:
            kwargs["detected"] = detected
        if fields is not unset:
            kwargs["fields"] = fields
        if notification_handles is not unset:
            kwargs["notification_handles"] = notification_handles
        if title is not unset:
            kwargs["title"] = title
        super().__init__(kwargs)



"""
Restore deleted dashboards returns "No Content" response
"""

from os import environ
from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.dashboards_api import DashboardsApi
from datadog_api_client.v1.model.dashboard_bulk_action_data import DashboardBulkActionData
from datadog_api_client.v1.model.dashboard_bulk_action_data_list import DashboardBulkActionDataList
from datadog_api_client.v1.model.dashboard_resource_type import DashboardResourceType
from datadog_api_client.v1.model.dashboard_restore_request import DashboardRestoreRequest

# there is a valid "dashboard" in the system
DASHBOARD_ID = environ["DASHBOARD_ID"]

body = DashboardRestoreRequest(
    data=DashboardBulkActionDataList(
        [
            DashboardBulkActionData(
                id=DASHBOARD_ID,
                type=DashboardResourceType.DASHBOARD,
            ),
        ]
    ),
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = DashboardsApi(api_client)
    api_instance.restore_dashboards(body=body)



"""
Update a webhook returns "OK" response
"""

from os import environ
from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.webhooks_integration_api import WebhooksIntegrationApi
from datadog_api_client.v1.model.webhooks_integration_update_request import WebhooksIntegrationUpdateRequest

# there is a valid "webhook" in the system
WEBHOOK_NAME = environ["WEBHOOK_NAME"]

body = WebhooksIntegrationUpdateRequest(
    url="https://example.com/webhook-updated",
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = WebhooksIntegrationApi(api_client)
    response = api_instance.update_webhooks_integration(webhook_name=WEBHOOK_NAME, body=body)

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.role_update_attributes import RoleUpdateAttributes
    from datadog_api_client.v2.model.role_relationships import RoleRelationships
    from datadog_api_client.v2.model.roles_type import RolesType


class RoleUpdateData(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.role_update_attributes import RoleUpdateAttributes
        from datadog_api_client.v2.model.role_relationships import RoleRelationships
        from datadog_api_client.v2.model.roles_type import RolesType

        return {
            "attributes": (RoleUpdateAttributes,),
            "id": (str,),
            "relationships": (RoleRelationships,),
            "type": (RolesType,),
        }

    attribute_map = {
        "attributes": "attributes",
        "id": "id",
        "relationships": "relationships",
        "type": "type",
    }

    def __init__(
        self_,
        attributes: RoleUpdateAttributes,
        id: str,
        type: RolesType,
        relationships: Union[RoleRelationships, UnsetType] = unset,
        **kwargs,
    ):
        """
        Data related to the update of a role.

        :param attributes: Attributes of the role.
        :type attributes: RoleUpdateAttributes

        :param id: The unique identifier of the role.
        :type id: str

        :param relationships: Relationships of the role object.
        :type relationships: RoleRelationships, optional

        :param type: Roles type.
        :type type: RolesType
        """
        if relationships is not unset:
            kwargs["relationships"] = relationships
        super().__init__(kwargs)

        self_.attributes = attributes
        self_.id = id
        self_.type = type



"""
Paginate monthly usage attribution
"""

from datetime import datetime
from dateutil.relativedelta import relativedelta
from os import environ
from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.usage_metering_api import UsageMeteringApi
from datadog_api_client.v1.model.monthly_usage_attribution_supported_metrics import (
    MonthlyUsageAttributionSupportedMetrics,
)

# there is a valid "monthly_usage_attribution" response
MONTHLY_USAGE_ATTRIBUTION_METADATA_PAGINATION_NEXT_RECORD_ID = environ[
    "MONTHLY_USAGE_ATTRIBUTION_METADATA_PAGINATION_NEXT_RECORD_ID"
]

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = UsageMeteringApi(api_client)
    response = api_instance.get_monthly_usage_attribution(
        start_month=(datetime.now() + relativedelta(days=-3)),
        fields=MonthlyUsageAttributionSupportedMetrics.INFRA_HOST_USAGE,
        next_record_id=MONTHLY_USAGE_ATTRIBUTION_METADATA_PAGINATION_NEXT_RECORD_ID,
    )

    print(response)



import os

import pytest

from datadog_api_client.api_client import ThreadedApiClient
from datadog_api_client.configuration import Configuration
from datadog_api_client.v1.api import dashboards_api


def test_basic():
    if os.getenv("RECORD", "false").lower() != "none":
        pytest.skip("Integration test")
    configuration = Configuration(return_http_data_only=False)
    configuration.api_key["apiKeyAuth"] = os.getenv("DD_TEST_CLIENT_API_KEY", "fake")
    configuration.api_key["appKeyAuth"] = os.getenv("DD_TEST_CLIENT_APP_KEY", "fake")
    configuration.debug = os.getenv("DEBUG") in {"true", "1", "yes", "on"}
    if "DD_TEST_SITE" in os.environ:
        configuration.server_index = 2
        configuration.server_variables["site"] = os.environ["DD_TEST_SITE"]

    with ThreadedApiClient(configuration) as api_client:
        api_instance = dashboards_api.DashboardsApi(api_client)
        thread = api_instance.list_dashboards()
        _, code, headers = thread.get()
        assert code == 200
        assert headers["Content-Type"] == "application/json"



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class HTTPLogItem(ModelNormal):
    @cached_property
    def additional_properties_type(_):
        return (str,)

    @cached_property
    def openapi_types(_):
        return {
            "ddsource": (str,),
            "ddtags": (str,),
            "hostname": (str,),
            "message": (str,),
            "service": (str,),
        }

    attribute_map = {
        "ddsource": "ddsource",
        "ddtags": "ddtags",
        "hostname": "hostname",
        "message": "message",
        "service": "service",
    }

    def __init__(
        self_,
        message: str,
        ddsource: Union[str, UnsetType] = unset,
        ddtags: Union[str, UnsetType] = unset,
        hostname: Union[str, UnsetType] = unset,
        service: Union[str, UnsetType] = unset,
        **kwargs,
    ):
        """
        Logs that are sent over HTTP.

        :param ddsource: The integration name associated with your log: the technology from which the log originated.
            When it matches an integration name, Datadog automatically installs the corresponding parsers and facets.
            See `reserved attributes <https://docs.datadoghq.com/logs/log_collection/#reserved-attributes>`_.
        :type ddsource: str, optional

        :param ddtags: Tags associated with your logs.
        :type ddtags: str, optional

        :param hostname: The name of the originating host of the log.
        :type hostname: str, optional

        :param message: The message `reserved attribute <https://docs.datadoghq.com/logs/log_collection/#reserved-attributes>`_
            of your log. By default, Datadog ingests the value of the message attribute as the body of the log entry.
            That value is then highlighted and displayed in the Logstream, where it is indexed for full text search.
        :type message: str

        :param service: The name of the application or service generating the log events.
            It is used to switch from Logs to APM, so make sure you define the same value when you use both products.
            See `reserved attributes <https://docs.datadoghq.com/logs/log_collection/#reserved-attributes>`_.
        :type service: str, optional
        """
        if ddsource is not unset:
            kwargs["ddsource"] = ddsource
        if ddtags is not unset:
            kwargs["ddtags"] = ddtags
        if hostname is not unset:
            kwargs["hostname"] = hostname
        if service is not unset:
            kwargs["service"] = service
        super().__init__(kwargs)

        self_.message = message



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class SLOTypeNumeric(ModelSimple):
    """
    A numeric representation of the type of the service level objective (`0` for
        monitor, `1` for metric). Always included in service level objective responses.
        Ignored in create/update requests.

    :param value: Must be one of [0, 1].
    :type value: int
    """

    allowed_values = {
        0,
        1,
    }
    MONITOR: ClassVar["SLOTypeNumeric"]
    METRIC: ClassVar["SLOTypeNumeric"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (int,),
        }


SLOTypeNumeric.MONITOR = SLOTypeNumeric(0)
SLOTypeNumeric.METRIC = SLOTypeNumeric(1)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)


class EventsQueryGroupBys(ModelSimple):
    """
    The list of facets on which to split results.


    :type value: [EventsGroupBy]
    """

    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.events_group_by import EventsGroupBy

        return {
            "value": ([EventsGroupBy],),
        }



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class ServiceDefinitionV1Org(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "application": (str,),
            "team": (str,),
        }

    attribute_map = {
        "application": "application",
        "team": "team",
    }

    def __init__(self_, application: Union[str, UnsetType] = unset, team: Union[str, UnsetType] = unset, **kwargs):
        """
        Org related information about the service.

        :param application: App feature this service supports.
        :type application: str, optional

        :param team: Team that owns the service.
        :type team: str, optional
        """
        if application is not unset:
            kwargs["application"] = application
        if team is not unset:
            kwargs["team"] = team
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    none_type,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.slo_history_metrics_series_metadata_unit import SLOHistoryMetricsSeriesMetadataUnit


class SLOHistoryMetricsSeriesMetadata(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.slo_history_metrics_series_metadata_unit import (
            SLOHistoryMetricsSeriesMetadataUnit,
        )

        return {
            "aggr": (str,),
            "expression": (str,),
            "metric": (str,),
            "query_index": (int,),
            "scope": (str,),
            "unit": ([SLOHistoryMetricsSeriesMetadataUnit, none_type], none_type),
        }

    attribute_map = {
        "aggr": "aggr",
        "expression": "expression",
        "metric": "metric",
        "query_index": "query_index",
        "scope": "scope",
        "unit": "unit",
    }

    def __init__(
        self_,
        aggr: Union[str, UnsetType] = unset,
        expression: Union[str, UnsetType] = unset,
        metric: Union[str, UnsetType] = unset,
        query_index: Union[int, UnsetType] = unset,
        scope: Union[str, UnsetType] = unset,
        unit: Union[List[SLOHistoryMetricsSeriesMetadataUnit], none_type, UnsetType] = unset,
        **kwargs,
    ):
        """
        Query metadata.

        :param aggr: Query aggregator function.
        :type aggr: str, optional

        :param expression: Query expression.
        :type expression: str, optional

        :param metric: Query metric used.
        :type metric: str, optional

        :param query_index: Query index from original combined query.
        :type query_index: int, optional

        :param scope: Query scope.
        :type scope: str, optional

        :param unit: An array of metric units that contains up to two unit objects.
            For example, bytes represents one unit object and bytes per second represents two unit objects.
            If a metric query only has one unit object, the second array element is null.
        :type unit: [SLOHistoryMetricsSeriesMetadataUnit, none_type], none_type, optional
        """
        if aggr is not unset:
            kwargs["aggr"] = aggr
        if expression is not unset:
            kwargs["expression"] = expression
        if metric is not unset:
            kwargs["metric"] = metric
        if query_index is not unset:
            kwargs["query_index"] = query_index
        if scope is not unset:
            kwargs["scope"] = scope
        if unit is not unset:
            kwargs["unit"] = unit
        super().__init__(kwargs)



"""
Create a custom variable returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.webhooks_integration_api import WebhooksIntegrationApi
from datadog_api_client.v1.model.webhooks_integration_custom_variable import WebhooksIntegrationCustomVariable

body = WebhooksIntegrationCustomVariable(
    is_secret=True,
    name="EXAMPLECREATEACUSTOMVARIABLERETURNSOKRESPONSE",
    value="CUSTOM_VARIABLE_VALUE",
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = WebhooksIntegrationApi(api_client)
    response = api_instance.create_webhooks_integration_custom_variable(body=body)

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.events_query_filter import EventsQueryFilter
    from datadog_api_client.v2.model.events_query_options import EventsQueryOptions
    from datadog_api_client.v2.model.events_request_page import EventsRequestPage
    from datadog_api_client.v2.model.events_sort import EventsSort


class EventsListRequest(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.events_query_filter import EventsQueryFilter
        from datadog_api_client.v2.model.events_query_options import EventsQueryOptions
        from datadog_api_client.v2.model.events_request_page import EventsRequestPage
        from datadog_api_client.v2.model.events_sort import EventsSort

        return {
            "filter": (EventsQueryFilter,),
            "options": (EventsQueryOptions,),
            "page": (EventsRequestPage,),
            "sort": (EventsSort,),
        }

    attribute_map = {
        "filter": "filter",
        "options": "options",
        "page": "page",
        "sort": "sort",
    }

    def __init__(
        self_,
        filter: Union[EventsQueryFilter, UnsetType] = unset,
        options: Union[EventsQueryOptions, UnsetType] = unset,
        page: Union[EventsRequestPage, UnsetType] = unset,
        sort: Union[EventsSort, UnsetType] = unset,
        **kwargs,
    ):
        """
        The object sent with the request to retrieve a list of events from your organization.

        :param filter: The search and filter query settings.
        :type filter: EventsQueryFilter, optional

        :param options: The global query options that are used. Either provide a timezone or a time offset but not both,
            otherwise the query fails.
        :type options: EventsQueryOptions, optional

        :param page: Pagination settings.
        :type page: EventsRequestPage, optional

        :param sort: The sort parameters when querying events.
        :type sort: EventsSort, optional
        """
        if filter is not unset:
            kwargs["filter"] = filter
        if options is not unset:
            kwargs["options"] = options
        if page is not unset:
            kwargs["page"] = page
        if sort is not unset:
            kwargs["sort"] = sort
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class SyntheticsBasicAuthOauthTokenApiAuthentication(ModelSimple):
    """
    Type of token to use when performing the authentication.

    :param value: Must be one of ["header", "body"].
    :type value: str
    """

    allowed_values = {
        "header",
        "body",
    }
    HEADER: ClassVar["SyntheticsBasicAuthOauthTokenApiAuthentication"]
    BODY: ClassVar["SyntheticsBasicAuthOauthTokenApiAuthentication"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


SyntheticsBasicAuthOauthTokenApiAuthentication.HEADER = SyntheticsBasicAuthOauthTokenApiAuthentication("header")
SyntheticsBasicAuthOauthTokenApiAuthentication.BODY = SyntheticsBasicAuthOauthTokenApiAuthentication("body")



"""
Create a new incident team returns "CREATED" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.incident_teams_api import IncidentTeamsApi
from datadog_api_client.v2.model.incident_team_create_attributes import IncidentTeamCreateAttributes
from datadog_api_client.v2.model.incident_team_create_data import IncidentTeamCreateData
from datadog_api_client.v2.model.incident_team_create_request import IncidentTeamCreateRequest
from datadog_api_client.v2.model.incident_team_type import IncidentTeamType

body = IncidentTeamCreateRequest(
    data=IncidentTeamCreateData(
        type=IncidentTeamType.TEAMS,
        attributes=IncidentTeamCreateAttributes(
            name="Example-Create_a_new_incident_team_returns_CREATED_response",
        ),
    ),
)

configuration = Configuration()
configuration.unstable_operations["create_incident_team"] = True
with ApiClient(configuration) as api_client:
    api_instance = IncidentTeamsApi(api_client)
    response = api_instance.create_incident_team(body=body)

    print(response)



"""
Update a dashboard list returns "OK" response
"""

from os import environ
from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.dashboard_lists_api import DashboardListsApi
from datadog_api_client.v1.model.dashboard_list import DashboardList

# there is a valid "dashboard_list" in the system
DASHBOARD_LIST_ID = environ["DASHBOARD_LIST_ID"]

body = DashboardList(
    name="updated Example-Update_a_dashboard_list_returns_OK_response",
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = DashboardListsApi(api_client)
    response = api_instance.update_dashboard_list(list_id=int(DASHBOARD_LIST_ID), body=body)

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.incident_integration_metadata_type import IncidentIntegrationMetadataType


class RelationshipToIncidentIntegrationMetadataData(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.incident_integration_metadata_type import IncidentIntegrationMetadataType

        return {
            "id": (str,),
            "type": (IncidentIntegrationMetadataType,),
        }

    attribute_map = {
        "id": "id",
        "type": "type",
    }

    def __init__(self_, id: str, type: IncidentIntegrationMetadataType, **kwargs):
        """
        A relationship reference for an integration metadata object.

        :param id: A unique identifier that represents the integration metadata.
        :type id: str

        :param type: Integration metadata resource type.
        :type type: IncidentIntegrationMetadataType
        """
        super().__init__(kwargs)

        self_.id = id
        self_.type = type



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.widget_text_align import WidgetTextAlign
    from datadog_api_client.v1.model.free_text_widget_definition_type import FreeTextWidgetDefinitionType


class FreeTextWidgetDefinition(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.widget_text_align import WidgetTextAlign
        from datadog_api_client.v1.model.free_text_widget_definition_type import FreeTextWidgetDefinitionType

        return {
            "color": (str,),
            "font_size": (str,),
            "text": (str,),
            "text_align": (WidgetTextAlign,),
            "type": (FreeTextWidgetDefinitionType,),
        }

    attribute_map = {
        "color": "color",
        "font_size": "font_size",
        "text": "text",
        "text_align": "text_align",
        "type": "type",
    }

    def __init__(
        self_,
        text: str,
        type: FreeTextWidgetDefinitionType,
        color: Union[str, UnsetType] = unset,
        font_size: Union[str, UnsetType] = unset,
        text_align: Union[WidgetTextAlign, UnsetType] = unset,
        **kwargs,
    ):
        """
        Free text is a widget that allows you to add headings to your screenboard. Commonly used to state the overall purpose of the dashboard. Only available on FREE layout dashboards.

        :param color: Color of the text.
        :type color: str, optional

        :param font_size: Size of the text.
        :type font_size: str, optional

        :param text: Text to display.
        :type text: str

        :param text_align: How to align the text on the widget.
        :type text_align: WidgetTextAlign, optional

        :param type: Type of the free text widget.
        :type type: FreeTextWidgetDefinitionType
        """
        if color is not unset:
            kwargs["color"] = color
        if font_size is not unset:
            kwargs["font_size"] = font_size
        if text_align is not unset:
            kwargs["text_align"] = text_align
        super().__init__(kwargs)

        self_.text = text
        self_.type = type



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class WidgetMonitorSummarySort(ModelSimple):
    """
    Widget sorting methods.

    :param value: Must be one of ["name", "group", "status", "tags", "triggered", "group,asc", "group,desc", "name,asc", "name,desc", "status,asc", "status,desc", "tags,asc", "tags,desc", "triggered,asc", "triggered,desc", "priority,asc", "priority,desc"].
    :type value: str
    """

    allowed_values = {
        "name",
        "group",
        "status",
        "tags",
        "triggered",
        "group,asc",
        "group,desc",
        "name,asc",
        "name,desc",
        "status,asc",
        "status,desc",
        "tags,asc",
        "tags,desc",
        "triggered,asc",
        "triggered,desc",
        "priority,asc",
        "priority,desc",
    }
    NAME: ClassVar["WidgetMonitorSummarySort"]
    GROUP: ClassVar["WidgetMonitorSummarySort"]
    STATUS: ClassVar["WidgetMonitorSummarySort"]
    TAGS: ClassVar["WidgetMonitorSummarySort"]
    TRIGGERED: ClassVar["WidgetMonitorSummarySort"]
    GROUP_ASCENDING: ClassVar["WidgetMonitorSummarySort"]
    GROUP_DESCENDING: ClassVar["WidgetMonitorSummarySort"]
    NAME_ASCENDING: ClassVar["WidgetMonitorSummarySort"]
    NAME_DESCENDING: ClassVar["WidgetMonitorSummarySort"]
    STATUS_ASCENDING: ClassVar["WidgetMonitorSummarySort"]
    STATUS_DESCENDING: ClassVar["WidgetMonitorSummarySort"]
    TAGS_ASCENDING: ClassVar["WidgetMonitorSummarySort"]
    TAGS_DESCENDING: ClassVar["WidgetMonitorSummarySort"]
    TRIGGERED_ASCENDING: ClassVar["WidgetMonitorSummarySort"]
    TRIGGERED_DESCENDING: ClassVar["WidgetMonitorSummarySort"]
    PRIORITY_ASCENDING: ClassVar["WidgetMonitorSummarySort"]
    PRIORITY_DESCENDING: ClassVar["WidgetMonitorSummarySort"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


WidgetMonitorSummarySort.NAME = WidgetMonitorSummarySort("name")
WidgetMonitorSummarySort.GROUP = WidgetMonitorSummarySort("group")
WidgetMonitorSummarySort.STATUS = WidgetMonitorSummarySort("status")
WidgetMonitorSummarySort.TAGS = WidgetMonitorSummarySort("tags")
WidgetMonitorSummarySort.TRIGGERED = WidgetMonitorSummarySort("triggered")
WidgetMonitorSummarySort.GROUP_ASCENDING = WidgetMonitorSummarySort("group,asc")
WidgetMonitorSummarySort.GROUP_DESCENDING = WidgetMonitorSummarySort("group,desc")
WidgetMonitorSummarySort.NAME_ASCENDING = WidgetMonitorSummarySort("name,asc")
WidgetMonitorSummarySort.NAME_DESCENDING = WidgetMonitorSummarySort("name,desc")
WidgetMonitorSummarySort.STATUS_ASCENDING = WidgetMonitorSummarySort("status,asc")
WidgetMonitorSummarySort.STATUS_DESCENDING = WidgetMonitorSummarySort("status,desc")
WidgetMonitorSummarySort.TAGS_ASCENDING = WidgetMonitorSummarySort("tags,asc")
WidgetMonitorSummarySort.TAGS_DESCENDING = WidgetMonitorSummarySort("tags,desc")
WidgetMonitorSummarySort.TRIGGERED_ASCENDING = WidgetMonitorSummarySort("triggered,asc")
WidgetMonitorSummarySort.TRIGGERED_DESCENDING = WidgetMonitorSummarySort("triggered,desc")
WidgetMonitorSummarySort.PRIORITY_ASCENDING = WidgetMonitorSummarySort("priority,asc")
WidgetMonitorSummarySort.PRIORITY_DESCENDING = WidgetMonitorSummarySort("priority,desc")



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.incident_service_response_data import IncidentServiceResponseData
    from datadog_api_client.v2.model.incident_service_included_items import IncidentServiceIncludedItems
    from datadog_api_client.v2.model.user import User


class IncidentServiceResponse(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.incident_service_response_data import IncidentServiceResponseData
        from datadog_api_client.v2.model.incident_service_included_items import IncidentServiceIncludedItems

        return {
            "data": (IncidentServiceResponseData,),
            "included": ([IncidentServiceIncludedItems],),
        }

    attribute_map = {
        "data": "data",
        "included": "included",
    }
    read_only_vars = {
        "included",
    }

    def __init__(
        self_,
        data: IncidentServiceResponseData,
        included: Union[List[Union[IncidentServiceIncludedItems, User]], UnsetType] = unset,
        **kwargs,
    ):
        """
        Response with an incident service payload.

        :param data: Incident Service data from responses.
        :type data: IncidentServiceResponseData

        :param included: Included objects from relationships.
        :type included: [IncidentServiceIncludedItems], optional
        """
        if included is not unset:
            kwargs["included"] = included
        super().__init__(kwargs)

        self_.data = data



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class TimeseriesWidgetLegendColumn(ModelSimple):
    """
    Legend column.

    :param value: Must be one of ["value", "avg", "sum", "min", "max"].
    :type value: str
    """

    allowed_values = {
        "value",
        "avg",
        "sum",
        "min",
        "max",
    }
    VALUE: ClassVar["TimeseriesWidgetLegendColumn"]
    AVG: ClassVar["TimeseriesWidgetLegendColumn"]
    SUM: ClassVar["TimeseriesWidgetLegendColumn"]
    MIN: ClassVar["TimeseriesWidgetLegendColumn"]
    MAX: ClassVar["TimeseriesWidgetLegendColumn"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


TimeseriesWidgetLegendColumn.VALUE = TimeseriesWidgetLegendColumn("value")
TimeseriesWidgetLegendColumn.AVG = TimeseriesWidgetLegendColumn("avg")
TimeseriesWidgetLegendColumn.SUM = TimeseriesWidgetLegendColumn("sum")
TimeseriesWidgetLegendColumn.MIN = TimeseriesWidgetLegendColumn("min")
TimeseriesWidgetLegendColumn.MAX = TimeseriesWidgetLegendColumn("max")



"""
Create a new dashboard with hostmap widget
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.dashboards_api import DashboardsApi
from datadog_api_client.v1.model.dashboard import Dashboard
from datadog_api_client.v1.model.dashboard_layout_type import DashboardLayoutType
from datadog_api_client.v1.model.host_map_request import HostMapRequest
from datadog_api_client.v1.model.host_map_widget_definition import HostMapWidgetDefinition
from datadog_api_client.v1.model.host_map_widget_definition_requests import HostMapWidgetDefinitionRequests
from datadog_api_client.v1.model.host_map_widget_definition_style import HostMapWidgetDefinitionStyle
from datadog_api_client.v1.model.host_map_widget_definition_type import HostMapWidgetDefinitionType
from datadog_api_client.v1.model.widget import Widget
from datadog_api_client.v1.model.widget_layout import WidgetLayout
from datadog_api_client.v1.model.widget_node_type import WidgetNodeType
from datadog_api_client.v1.model.widget_text_align import WidgetTextAlign

body = Dashboard(
    title="Example-Create_a_new_dashboard_with_hostmap_widget",
    description=None,
    widgets=[
        Widget(
            layout=WidgetLayout(
                x=0,
                y=0,
                width=47,
                height=22,
            ),
            definition=HostMapWidgetDefinition(
                title="",
                title_size="16",
                title_align=WidgetTextAlign.LEFT,
                type=HostMapWidgetDefinitionType.HOSTMAP,
                requests=HostMapWidgetDefinitionRequests(
                    fill=HostMapRequest(
                        q="avg:system.cpu.user{*} by {host}",
                    ),
                ),
                node_type=WidgetNodeType.HOST,
                no_metric_hosts=True,
                no_group_hosts=True,
                style=HostMapWidgetDefinitionStyle(
                    palette="green_to_orange",
                    palette_flip=False,
                ),
            ),
        ),
    ],
    template_variables=[],
    layout_type=DashboardLayoutType.FREE,
    is_read_only=False,
    notify_list=[],
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = DashboardsApi(api_client)
    response = api_instance.create_dashboard(body=body)

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class HourlyUsageType(ModelSimple):
    """
    Usage type that is being measured.

    :param value: Must be one of ["app_sec_host_count", "observability_pipelines_bytes_processed", "lambda_traced_invocations_count"].
    :type value: str
    """

    allowed_values = {
        "app_sec_host_count",
        "observability_pipelines_bytes_processed",
        "lambda_traced_invocations_count",
    }
    APP_SEC_HOST_COUNT: ClassVar["HourlyUsageType"]
    OBSERVABILITY_PIPELINES_BYTES_PROCESSSED: ClassVar["HourlyUsageType"]
    LAMBDA_TRACED_INVOCATIONS_COUNT: ClassVar["HourlyUsageType"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


HourlyUsageType.APP_SEC_HOST_COUNT = HourlyUsageType("app_sec_host_count")
HourlyUsageType.OBSERVABILITY_PIPELINES_BYTES_PROCESSSED = HourlyUsageType("observability_pipelines_bytes_processed")
HourlyUsageType.LAMBDA_TRACED_INVOCATIONS_COUNT = HourlyUsageType("lambda_traced_invocations_count")



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.query_formula import QueryFormula
    from datadog_api_client.v2.model.timeseries_formula_request_queries import TimeseriesFormulaRequestQueries


class TimeseriesFormulaRequestAttributes(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.query_formula import QueryFormula
        from datadog_api_client.v2.model.timeseries_formula_request_queries import TimeseriesFormulaRequestQueries

        return {
            "formulas": ([QueryFormula],),
            "_from": (int,),
            "interval": (int,),
            "queries": (TimeseriesFormulaRequestQueries,),
            "to": (int,),
        }

    attribute_map = {
        "formulas": "formulas",
        "_from": "from",
        "interval": "interval",
        "queries": "queries",
        "to": "to",
    }

    def __init__(
        self_,
        _from: int,
        queries: TimeseriesFormulaRequestQueries,
        to: int,
        formulas: Union[List[QueryFormula], UnsetType] = unset,
        interval: Union[int, UnsetType] = unset,
        **kwargs,
    ):
        """
        The object describing a timeseries formula request.

        :param formulas: List of formulas to be calculated and returned as responses.
        :type formulas: [QueryFormula], optional

        :param _from: Start date (inclusive) of the query in milliseconds since the Unix epoch.
        :type _from: int

        :param interval: A time interval in milliseconds.
            May be overridden by a larger interval if the query would result in
            too many points for the specified timeframe.
            Defaults to a reasonable interval for the given timeframe.
        :type interval: int, optional

        :param queries: List of queries to be run and used as inputs to the formulas.
        :type queries: TimeseriesFormulaRequestQueries

        :param to: End date (exclusive) of the query in milliseconds since the Unix epoch.
        :type to: int
        """
        if formulas is not unset:
            kwargs["formulas"] = formulas
        if interval is not unset:
            kwargs["interval"] = interval
        super().__init__(kwargs)

        self_._from = _from
        self_.queries = queries
        self_.to = to



"""
Update an SLO returns "OK" response
"""

from os import environ
from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.service_level_objectives_api import ServiceLevelObjectivesApi
from datadog_api_client.v1.model.service_level_objective import ServiceLevelObjective
from datadog_api_client.v1.model.service_level_objective_query import ServiceLevelObjectiveQuery
from datadog_api_client.v1.model.slo_threshold import SLOThreshold
from datadog_api_client.v1.model.slo_timeframe import SLOTimeframe
from datadog_api_client.v1.model.slo_type import SLOType

# there is a valid "slo" in the system
SLO_DATA_0_ID = environ["SLO_DATA_0_ID"]
SLO_DATA_0_NAME = environ["SLO_DATA_0_NAME"]

body = ServiceLevelObjective(
    type=SLOType.METRIC,
    name=SLO_DATA_0_NAME,
    thresholds=[
        SLOThreshold(
            target=97.0,
            timeframe=SLOTimeframe.SEVEN_DAYS,
            warning=98.0,
        ),
    ],
    timeframe=SLOTimeframe.SEVEN_DAYS,
    target_threshold=97.0,
    warning_threshold=98.0,
    query=ServiceLevelObjectiveQuery(
        numerator="sum:httpservice.hits{code:2xx}.as_count()",
        denominator="sum:httpservice.hits{!code:3xx}.as_count()",
    ),
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = ServiceLevelObjectivesApi(api_client)
    response = api_instance.update_slo(slo_id=SLO_DATA_0_ID, body=body)

    print(response)



"""
Modify the triage assignee of a security signal returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.security_monitoring_api import SecurityMonitoringApi
from datadog_api_client.v1.model.signal_assignee_update_request import SignalAssigneeUpdateRequest

body = SignalAssigneeUpdateRequest(
    assignee="773b045d-ccf8-4808-bd3b-955ef6a8c940",
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = SecurityMonitoringApi(api_client)
    response = api_instance.edit_security_monitoring_signal_assignee(
        signal_id="AQAAAYDiB_Ol8PbzFAAAAABBWURpQl9PbEFBQU0yeXhGTG9ZV2JnQUE", body=body
    )

    print(response)



"""
Get an index returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.logs_indexes_api import LogsIndexesApi

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = LogsIndexesApi(api_client)
    response = api_instance.get_logs_index(
        name="name",
    )

    print(response)



"""
Get a user organization returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.users_api import UsersApi

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = UsersApi(api_client)
    response = api_instance.list_user_organizations(
        user_id="00000000-0000-9999-0000-000000000000",
    )

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)


class SyntheticsRestrictedRoles(ModelSimple):
    """
    A list of role identifiers that can be pulled from the Roles API, for restricting read and write access.


    :type value: [str]
    """

    @cached_property
    def openapi_types(_):
        return {
            "value": ([str],),
        }



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.aws_tag_filter import AWSTagFilter


class AWSTagFilterListResponse(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.aws_tag_filter import AWSTagFilter

        return {
            "filters": ([AWSTagFilter],),
        }

    attribute_map = {
        "filters": "filters",
    }

    def __init__(self_, filters: Union[List[AWSTagFilter], UnsetType] = unset, **kwargs):
        """
        An array of tag filter rules by ``namespace`` and tag filter string.

        :param filters: An array of tag filters.
        :type filters: [AWSTagFilter], optional
        """
        if filters is not unset:
            kwargs["filters"] = filters
        super().__init__(kwargs)



"""
Aggregate compute events returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.logs_api import LogsApi
from datadog_api_client.v2.model.logs_aggregate_request import LogsAggregateRequest
from datadog_api_client.v2.model.logs_aggregation_function import LogsAggregationFunction
from datadog_api_client.v2.model.logs_compute import LogsCompute
from datadog_api_client.v2.model.logs_compute_type import LogsComputeType
from datadog_api_client.v2.model.logs_query_filter import LogsQueryFilter

body = LogsAggregateRequest(
    compute=[
        LogsCompute(
            aggregation=LogsAggregationFunction.COUNT,
            interval="5m",
            type=LogsComputeType.TIMESERIES,
        ),
    ],
    filter=LogsQueryFilter(
        _from="now-15m",
        indexes=[
            "main",
        ],
        query="*",
        to="now",
    ),
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = LogsApi(api_client)
    response = api_instance.aggregate_logs(body=body)

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
)


class LogsArchiveIntegrationAzure(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "client_id": (str,),
            "tenant_id": (str,),
        }

    attribute_map = {
        "client_id": "client_id",
        "tenant_id": "tenant_id",
    }

    def __init__(self_, client_id: str, tenant_id: str, **kwargs):
        """
        The Azure archive's integration destination.

        :param client_id: A client ID.
        :type client_id: str

        :param tenant_id: A tenant ID.
        :type tenant_id: str
        """
        super().__init__(kwargs)

        self_.client_id = client_id
        self_.tenant_id = tenant_id



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.security_monitoring_filter_action import SecurityMonitoringFilterAction


class SecurityMonitoringFilter(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.security_monitoring_filter_action import SecurityMonitoringFilterAction

        return {
            "action": (SecurityMonitoringFilterAction,),
            "query": (str,),
        }

    attribute_map = {
        "action": "action",
        "query": "query",
    }

    def __init__(
        self_,
        action: Union[SecurityMonitoringFilterAction, UnsetType] = unset,
        query: Union[str, UnsetType] = unset,
        **kwargs,
    ):
        """
        The rule's suppression filter.

        :param action: The type of filtering action.
        :type action: SecurityMonitoringFilterAction, optional

        :param query: Query for selecting logs to apply the filtering action.
        :type query: str, optional
        """
        if action is not unset:
            kwargs["action"] = action
        if query is not unset:
            kwargs["query"] = query
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class SecurityMonitoringRuleImpossibleTravelOptions(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "baseline_user_locations": (bool,),
        }

    attribute_map = {
        "baseline_user_locations": "baselineUserLocations",
    }

    def __init__(self_, baseline_user_locations: Union[bool, UnsetType] = unset, **kwargs):
        """
        Options on impossible travel rules.

        :param baseline_user_locations: If true, signals are suppressed for the first 24 hours. In that time, Datadog learns the user's regular
            access locations. This can be helpful to reduce noise and infer VPN usage or credentialed API access.
        :type baseline_user_locations: bool, optional
        """
        if baseline_user_locations is not unset:
            kwargs["baseline_user_locations"] = baseline_user_locations
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class UsageCIVisibilityHour(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "ci_pipeline_indexed_spans": (int,),
            "ci_test_indexed_spans": (int,),
            "ci_visibility_pipeline_committers": (int,),
            "ci_visibility_test_committers": (int,),
            "org_name": (str,),
            "public_id": (str,),
        }

    attribute_map = {
        "ci_pipeline_indexed_spans": "ci_pipeline_indexed_spans",
        "ci_test_indexed_spans": "ci_test_indexed_spans",
        "ci_visibility_pipeline_committers": "ci_visibility_pipeline_committers",
        "ci_visibility_test_committers": "ci_visibility_test_committers",
        "org_name": "org_name",
        "public_id": "public_id",
    }

    def __init__(
        self_,
        ci_pipeline_indexed_spans: Union[int, UnsetType] = unset,
        ci_test_indexed_spans: Union[int, UnsetType] = unset,
        ci_visibility_pipeline_committers: Union[int, UnsetType] = unset,
        ci_visibility_test_committers: Union[int, UnsetType] = unset,
        org_name: Union[str, UnsetType] = unset,
        public_id: Union[str, UnsetType] = unset,
        **kwargs,
    ):
        """
        CI visibility usage in a given hour.

        :param ci_pipeline_indexed_spans: The number of spans for pipelines in the queried hour.
        :type ci_pipeline_indexed_spans: int, optional

        :param ci_test_indexed_spans: The number of spans for tests in the queried hour.
        :type ci_test_indexed_spans: int, optional

        :param ci_visibility_pipeline_committers: Shows the total count of all active Git committers for Pipelines in the current month. A committer is active if they commit at least 3 times in a given month.
        :type ci_visibility_pipeline_committers: int, optional

        :param ci_visibility_test_committers: The total count of all active Git committers for tests in the current month. A committer is active if they commit at least 3 times in a given month.
        :type ci_visibility_test_committers: int, optional

        :param org_name: The organization name.
        :type org_name: str, optional

        :param public_id: The organization public ID.
        :type public_id: str, optional
        """
        if ci_pipeline_indexed_spans is not unset:
            kwargs["ci_pipeline_indexed_spans"] = ci_pipeline_indexed_spans
        if ci_test_indexed_spans is not unset:
            kwargs["ci_test_indexed_spans"] = ci_test_indexed_spans
        if ci_visibility_pipeline_committers is not unset:
            kwargs["ci_visibility_pipeline_committers"] = ci_visibility_pipeline_committers
        if ci_visibility_test_committers is not unset:
            kwargs["ci_visibility_test_committers"] = ci_visibility_test_committers
        if org_name is not unset:
            kwargs["org_name"] = org_name
        if public_id is not unset:
            kwargs["public_id"] = public_id
        super().__init__(kwargs)



from datadog_api_client.v1.api.aws_integration_api import AWSIntegrationApi
from datadog_api_client.v1.api.aws_logs_integration_api import AWSLogsIntegrationApi
from datadog_api_client.v1.api.authentication_api import AuthenticationApi
from datadog_api_client.v1.api.azure_integration_api import AzureIntegrationApi
from datadog_api_client.v1.api.dashboard_lists_api import DashboardListsApi
from datadog_api_client.v1.api.dashboards_api import DashboardsApi
from datadog_api_client.v1.api.downtimes_api import DowntimesApi
from datadog_api_client.v1.api.events_api import EventsApi
from datadog_api_client.v1.api.gcp_integration_api import GCPIntegrationApi
from datadog_api_client.v1.api.hosts_api import HostsApi
from datadog_api_client.v1.api.ip_ranges_api import IPRangesApi
from datadog_api_client.v1.api.key_management_api import KeyManagementApi
from datadog_api_client.v1.api.logs_api import LogsApi
from datadog_api_client.v1.api.logs_indexes_api import LogsIndexesApi
from datadog_api_client.v1.api.logs_pipelines_api import LogsPipelinesApi
from datadog_api_client.v1.api.metrics_api import MetricsApi
from datadog_api_client.v1.api.monitors_api import MonitorsApi
from datadog_api_client.v1.api.notebooks_api import NotebooksApi
from datadog_api_client.v1.api.organizations_api import OrganizationsApi
from datadog_api_client.v1.api.pager_duty_integration_api import PagerDutyIntegrationApi
from datadog_api_client.v1.api.security_monitoring_api import SecurityMonitoringApi
from datadog_api_client.v1.api.service_checks_api import ServiceChecksApi
from datadog_api_client.v1.api.service_level_objective_corrections_api import ServiceLevelObjectiveCorrectionsApi
from datadog_api_client.v1.api.service_level_objectives_api import ServiceLevelObjectivesApi
from datadog_api_client.v1.api.slack_integration_api import SlackIntegrationApi
from datadog_api_client.v1.api.snapshots_api import SnapshotsApi
from datadog_api_client.v1.api.synthetics_api import SyntheticsApi
from datadog_api_client.v1.api.tags_api import TagsApi
from datadog_api_client.v1.api.usage_metering_api import UsageMeteringApi
from datadog_api_client.v1.api.users_api import UsersApi
from datadog_api_client.v1.api.webhooks_integration_api import WebhooksIntegrationApi



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.user import User


class UserListResponse(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.user import User

        return {
            "users": ([User],),
        }

    attribute_map = {
        "users": "users",
    }

    def __init__(self_, users: Union[List[User], UnsetType] = unset, **kwargs):
        """
        Array of Datadog users for a given organization.

        :param users: Array of users.
        :type users: [User], optional
        """
        if users is not unset:
            kwargs["users"] = users
        super().__init__(kwargs)



"""
Create a distribution widget using a histogram request containing a formulas and functions events query
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.dashboards_api import DashboardsApi
from datadog_api_client.v1.model.dashboard import Dashboard
from datadog_api_client.v1.model.dashboard_layout_type import DashboardLayoutType
from datadog_api_client.v1.model.distribution_widget_definition import DistributionWidgetDefinition
from datadog_api_client.v1.model.distribution_widget_definition_type import DistributionWidgetDefinitionType
from datadog_api_client.v1.model.distribution_widget_histogram_request_type import (
    DistributionWidgetHistogramRequestType,
)
from datadog_api_client.v1.model.distribution_widget_request import DistributionWidgetRequest
from datadog_api_client.v1.model.distribution_widget_x_axis import DistributionWidgetXAxis
from datadog_api_client.v1.model.distribution_widget_y_axis import DistributionWidgetYAxis
from datadog_api_client.v1.model.formula_and_function_event_aggregation import FormulaAndFunctionEventAggregation
from datadog_api_client.v1.model.formula_and_function_event_query_definition import (
    FormulaAndFunctionEventQueryDefinition,
)
from datadog_api_client.v1.model.formula_and_function_event_query_definition_compute import (
    FormulaAndFunctionEventQueryDefinitionCompute,
)
from datadog_api_client.v1.model.formula_and_function_event_query_definition_search import (
    FormulaAndFunctionEventQueryDefinitionSearch,
)
from datadog_api_client.v1.model.formula_and_function_events_data_source import FormulaAndFunctionEventsDataSource
from datadog_api_client.v1.model.widget import Widget
from datadog_api_client.v1.model.widget_layout import WidgetLayout
from datadog_api_client.v1.model.widget_text_align import WidgetTextAlign

body = Dashboard(
    title="Example-Create_a_distribution_widget_using_a_histogram_request_containing_a_formulas_and_functions_events_qu",
    description="Example-Create_a_distribution_widget_using_a_histogram_request_containing_a_formulas_and_functions_events_qu",
    widgets=[
        Widget(
            definition=DistributionWidgetDefinition(
                title="Events Platform - Request latency HOP",
                title_size="16",
                title_align=WidgetTextAlign.LEFT,
                show_legend=False,
                type=DistributionWidgetDefinitionType.DISTRIBUTION,
                xaxis=DistributionWidgetXAxis(
                    max="auto",
                    include_zero=True,
                    scale="linear",
                    min="auto",
                ),
                yaxis=DistributionWidgetYAxis(
                    max="auto",
                    include_zero=True,
                    scale="linear",
                    min="auto",
                ),
                requests=[
                    DistributionWidgetRequest(
                        query=FormulaAndFunctionEventQueryDefinition(
                            search=FormulaAndFunctionEventQueryDefinitionSearch(
                                query="",
                            ),
                            data_source=FormulaAndFunctionEventsDataSource.EVENTS,
                            compute=FormulaAndFunctionEventQueryDefinitionCompute(
                                metric="@duration",
                                aggregation=FormulaAndFunctionEventAggregation.MIN,
                            ),
                            name="query1",
                            indexes=[
                                "*",
                            ],
                            group_by=[],
                        ),
                        request_type=DistributionWidgetHistogramRequestType.HISTOGRAM,
                    ),
                ],
            ),
            layout=WidgetLayout(
                x=0,
                y=0,
                width=4,
                height=2,
            ),
        ),
    ],
    layout_type=DashboardLayoutType.ORDERED,
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = DashboardsApi(api_client)
    response = api_instance.create_dashboard(body=body)

    print(response)



"""
Get a user invitation returns "OK" response
"""

from os import environ
from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.users_api import UsersApi

# the "user" has a "user_invitation"
USER_INVITATION_ID = environ["USER_INVITATION_ID"]

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = UsersApi(api_client)
    response = api_instance.get_invitation(
        user_invitation_uuid=USER_INVITATION_ID,
    )

    print(response)
